###
# Copyright 2016-2021 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

# -*- coding: utf-8 -*-
""" IscsiConfig Command for rdmc """

import json
import re
import time

import redfish.ris

try:
    from rdmc_helper import (
        BootOrderMissingEntriesError,
        Encryption,
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        NicMissingOrConfigurationError,
        ReturnCodes,
    )
except:
    from ilorest.rdmc_helper import (
        BootOrderMissingEntriesError,
        Encryption,
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        NicMissingOrConfigurationError,
        ReturnCodes,
    )


class IscsiConfigCommand:
    """Changes the iscsi configuration for the server that is currently logged in"""

    def __init__(self):
        self.ident = {
            "name": "iscsiconfig",
            "usage": None,
            "description": "Run without"
            " arguments for available NIC sources for iSCSI"
            " configuration.\n\texample: iscsiconfig\n\n\tDisplay"
            " the current iSCSI configuration:\n\texample: "
            "iscsiconfig --list\n\n\tSaving current iSCSI "
            "configuration to a file:\n\texample: iscsiconfig "
            "--list -f output.txt\n\n\tLoading iSCSI "
            "configurations from a file:\n\texample: iscsiconfig "
            "--modify output.txt\n\n\tIn order to add a NIC "
            'source to an iSCSI boot attempt you must run \n\t"'
            'iscsiconfig" without any paramters. This will '
            "display a list of NIC\n\tsources which are currently"
            " present in the system.\n\tAdding an iSCSI boot "
            "attempt:\n\texample: iscsiconfig --add 1\n\n\tIn "
            "order to delete an iSCSI boot attempt you must run"
            '\n\t"iscsiconfig --list" to view the currently '
            "configured attempts.\n\tOnce you find the attempt "
            "you want to delete simply pass the attempt\n\tnumber"
            " to the iSCSI delete function.\n\tDeleting an iSCSI "
            "boot attempt:\n\texample: iscsiconfig --delete 1",
            "summary": "Displays and configures the current iscsi settings.",
            "aliases": [],
            "auxcommands": [
                "GetCommand",
                "SetCommand",
                "SelectCommand",
                "RebootCommand",
                "LogoutCommand",
            ],
        }
        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()

    def run(self, line, help_disp=False):
        """Main iscsi configuration worker function

        :param line: string of arguments passed in
        :type line: str.
        """
        if help_disp:
            self.parser.print_help()
            return ReturnCodes.SUCCESS
        try:
            (options, args) = self.rdmc.rdmc_parse_arglist(self, line)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineErrorOPTS("")

        self.iscsiconfigurationvalidation(options)

        if self.rdmc.app.typepath.defs.isgen10:
            iscsipath = self.gencompatpaths(selector="HpeiSCSISoftwareInitiator.", rel=False)
            if "/settings" not in iscsipath:
                iscsisettingspath = iscsipath + "settings"
            else:
                iscsisettingspath = iscsipath
            bootpath = self.gencompatpaths(selector="HpeServerBootSettings.")
            bootpath = bootpath.replace("/settings", "")
        else:
            # TODO: update gencompats to handle the nesting of these links within the gen 9 version.
            if self.rdmc.app.typepath.defs.biospath[-1] == "/":
                iscsipath = self.rdmc.app.typepath.defs.biospath + "iScsi/"
                iscsisettingspath = self.rdmc.app.typepath.defs.biospath + "iScsi/settings/"
                bootpath = self.rdmc.app.typepath.defs.biospath + "Boot/"
            else:
                iscsipath = self.rdmc.app.typepath.defs.biospath + "/iScsi"
                iscsisettingspath = self.rdmc.app.typepath.defs.biospath + "/iScsi/settings"
                bootpath = self.rdmc.app.typepath.defs.biospath + "/Boot"

        if options.list:
            self.listoptionhelper(options, iscsipath, iscsisettingspath, bootpath)
        elif options.modify:
            self.modifyoptionhelper(options, iscsisettingspath)
        elif options.add:
            self.addoptionhelper(options, iscsipath, iscsisettingspath, bootpath)
        elif options.delete:
            self.deleteoptionhelper(options, iscsisettingspath)
        elif not args:
            self.defaultsoptionhelper(options, iscsipath, bootpath)
        else:
            if len(args) < 2:
                self.iscsiconfigurationvalidation(options)
            else:
                raise InvalidCommandLineError(
                    "Invalid number of parameters. " "Iscsi configuration takes a maximum of 1 parameter."
                )

        if options.reboot:
            self.auxcommands["reboot"].run(options.reboot)

        self.cmdbase.logout_routine(self, options)
        # Return code
        return ReturnCodes.SUCCESS

    def gencompatpaths(self, selector=None, rel=False):
        """Helper function for finding gen compatible paths

        :param selector: the type selection for the get operation
        :type selector: str.
        :param rel: flag to tell select function to reload selected instance
        :type rel: boolean.
        :returns: returns urls
        """

        self.rdmc.app.select(selector=selector, path_refresh=rel)
        props = self.rdmc.app.getprops(skipnonsetting=False)
        for prop in props:
            name = prop.get("Name")
            if "current" in name.lower() or "pending" in name.lower():
                try:
                    path = prop.get("@odata.id")
                except:
                    self.rdmc.ui.error("URI path could not be found.")
        return path

    def get_enabled_only(self):
        enable_disable = dict()
        enable_nic_only = list()
        enable_nic = dict()
        final_enabled_list = list()
        import re

        bios = "/redfish/v1/systems/1/bios/"
        bios_dict = self.rdmc.app.get_handler(bios, silent=True, service=True).dict
        if "Attributes" in bios_dict:
            bios_dict_attributes = bios_dict["Attributes"].items()
        else:
            bios_dict_attributes = bios_dict.items()
        for attr, val in bios_dict_attributes:
            if "NetworkBoot" in str(val):
                enable_disable[attr] = val
        # Enabled NIC only listing
        for k, v in enable_disable.items():
            enable_nic_only.append(k)
            nics = re.findall("\\d+", k)
            enable_nic[k] = nics
            final_enabled_list.append(nics)

        return final_enabled_list

    def addoptionhelper(self, options, iscsipath, iscsisettingspath, bootpath):
        """Helper function to add option for iscsi

        :param options: command line options
        :type options: list.
        :param iscsipath: current iscsi path
        :type iscsipath: str.
        :param iscsisettingspath: current iscsi settings path
        :type iscsisettingspath: str.
        :param bootpath: current boot path
        :type bootpath: str.
        """
        devicealloc = list()
        self.auxcommands["select"].selectfunction("HpeBiosMapping.")
        pcisettingsmap = self.auxcommands["get"].getworkerfunction(
            "BiosPciSettingsMappings", options, results=True, uselist=True
        )

        for item in pcisettingsmap[0]["BiosPciSettingsMappings"]:
            if "Associations" in item:
                if "EmbNicEnable" in item["Associations"] or "EmbNicConfig" in item["Associations"]:
                    _ = [devicealloc.append(x) for x in item["Subinstances"]]

                if (
                    re.match("FlexLom[0-9]+Enable", item["Associations"][0])
                    or re.match("PciSlot[0-9]+Enable", item["Associations"][0])
                    or re.match("Slot[0-9]+NicBoot[0-9]+", item["Associations"][0])
                ):
                    _ = [devicealloc.append(x) for x in item["Subinstances"]]

        foundlocation = False
        iscsibootsources = self.rawdatahandler(action="GET", silent=True, jsonflag=True, path=iscsisettingspath)
        count = 0
        attemptinstancenumber = self.bootattemptcounter(iscsibootsources[self.rdmc.app.typepath.defs.iscsisource])
        if self.rdmc.app.typepath.defs.isgen10:
            newpcilist = []
            self.auxcommands["select"].selectfunction("HpeServerPciDeviceCollection")
            pcideviceslist = next(
                iter(self.auxcommands["get"].getworkerfunction("Members", options, results=True, uselist=False)),
                None,
            )
            for device in pcideviceslist["Members"]:
                newpcilist.append(self.rdmc.app.get_handler(device["@odata.id"], silent=True).dict)
            pcideviceslist = newpcilist
        else:
            self.auxcommands["select"].selectfunction(["Collection."])
            pcideviceslist = next(
                iter(
                    self.auxcommands["get"].getworkerfunction(
                        "Items",
                        options,
                        results=True,
                        uselist=False,
                        filtervals=("MemberType", "HpServerPciDevice.*"),
                    )
                ),
                None,
            )["Items"]

        self.pcidevicehelper(devicealloc, iscsipath, bootpath, pcideviceslist)

        devicealloc_final = self.make_final_list(devicealloc, pcideviceslist)

        print_flag = False
        iscsi_list = self.listoptionhelper(options, iscsipath, iscsisettingspath, bootpath, print_flag)
        refined_list = list()
        for s in iscsi_list:
            for key, val in s.items():
                if key != "Not Added":
                    for k, v in val.items():
                        refined_list.append(v["iSCSINicSource"])
        enabled_only_bootsource = self.get_enabled_only()
        refined_devicealloc = list()
        for d in devicealloc:
            if "PreBootNetwork" in d["Associations"][0]:
                nic = d["Associations"][1]
            else:
                nic = d["Associations"][0]
            if "Storage" in nic:
                continue
            nics = re.findall("\\d+", nic)
            if nics in enabled_only_bootsource:
                refined_devicealloc.append(d)

        for item in iscsibootsources[self.rdmc.app.typepath.defs.iscsisource]:
            try:
                if not item[self.rdmc.app.typepath.defs.iscsiattemptinstance]:
                    nicsourcedata = refined_devicealloc[int(options.add) - 1]["Associations"]
                    if "PreBootNetwork" in nicsourcedata[0]:
                        nic_getting_added = nicsourcedata[1]
                    else:
                        nic_getting_added = nicsourcedata[0]
                    if nic_getting_added in refined_list:
                        self.rdmc.ui.printer("Warning: This NIC is already added to existing attempt\n")
                    iscsibootsources[self.rdmc.app.typepath.defs.iscsisource][count]["iSCSINicSource"] = (
                        nicsourcedata[1] if isinstance(nicsourcedata[0], dict) else nicsourcedata[0]
                    )

                    iscsibootsources[self.rdmc.app.typepath.defs.iscsisource][count][
                        self.rdmc.app.typepath.defs.iscsiattemptinstance
                    ] = attemptinstancenumber
                    iscsibootsources[self.rdmc.app.typepath.defs.iscsisource][count][
                        self.rdmc.app.typepath.defs.iscsiattemptname
                    ] = str(attemptinstancenumber)
                    foundlocation = True
                    break
            except Exception:
                raise NicMissingOrConfigurationError("Invalid input value for configuring NIC.")
            count += 1

        if foundlocation:
            self.rdmc.app.patch_handler(
                iscsisettingspath,
                iscsibootsources,
                optionalpassword=options.biospassword,
            )
        else:
            raise NicMissingOrConfigurationError("Failed to add NIC. All NICs" " have already been configured.")

    def make_final_list(self, devicealloc, pcideviceslist):
        final_devicealloc = []
        for item in devicealloc:
            if isinstance(item["Associations"][0], dict):
                listval = 1
            else:
                listval = 0
            if "Storage" not in item["Associations"][listval]:
                for pcidevice in pcideviceslist:
                    # if item["CorrelatableID"] == pcidevice["UEFIDevicePath"]:
                    final_devicealloc.append(item)
        return final_devicealloc

    def bootattemptcounter(self, bootsources):
        """Helper function to count the current boot entries for iscsi

        :param bootsources: current iscsi boot sources
        :type bootsources: list.
        """
        size = 0
        count = list()

        for item in bootsources:
            size += 1

            if item[self.rdmc.app.typepath.defs.iscsiattemptinstance]:
                count.append(int(item[self.rdmc.app.typepath.defs.iscsiattemptinstance]))

        if size == len(count):
            raise NicMissingOrConfigurationError("Failed to add NIC. All " "NICs have already been configured.")

        count.sort(key=None, reverse=False)

        if len(count) > 0:
            iterate = 0

            for i in range(1, size + 1, 1):
                if iterate < len(count) and i == count[iterate]:
                    iterate += 1
                else:
                    return iterate + 1
        else:
            return int(1)

    def deleteoptionhelper(self, options, iscsisettingspath):
        """Helper function to delete option for iscsi

        :param options: command line options
        :type options: list.
        :param iscsisettingspath: current iscsi settings path
        :type iscsisettingspath: string.
        """
        patch = None

        self.auxcommands["select"].selectfunction("HpBaseConfigs.")
        contents = self.rdmc.app.getprops(selector="BaseConfigs")

        for content in contents:
            for key in content["BaseConfigs"][0]["default"]:
                if key == self.rdmc.app.typepath.defs.iscsisource:
                    patch = content["BaseConfigs"][0]["default"][key]

        if not patch:
            raise NicMissingOrConfigurationError("Could not access Base Configurations.")

        self.validateinput(options=options, deleteoption=True)

        foundlocation = False
        iscsibootsources = self.rawdatahandler(action="GET", silent=True, jsonflag=False, path=iscsisettingspath)
        holdetag = iscsibootsources.getheader("etag")
        iscsibootsources = json.loads(iscsibootsources.read)

        try:
            count = 0
            for item in iscsibootsources[self.rdmc.app.typepath.defs.iscsisource]:
                if item[self.rdmc.app.typepath.defs.iscsiattemptinstance] == int(options.delete):
                    iscsibootsources[self.rdmc.app.typepath.defs.iscsisource][count] = patch[count]
                    foundlocation = True

                count += 1
        except Exception:
            raise NicMissingOrConfigurationError(
                "The NIC targeted for delete" " does not exist. The request for " "delete could not be completed."
            )

        if foundlocation:
            self.rdmc.app.put_handler(
                iscsisettingspath,
                iscsibootsources,
                optionalpassword=options.biospassword,
                headers={"if-Match": holdetag},
            )
            self.rdmc.app.get_handler(iscsisettingspath, silent=True)
        else:
            raise NicMissingOrConfigurationError("The given attempt instance does not exist.")

    def listoptionhelper(self, options, iscsipath, iscsisettingspath, bootpath, print_flag=True):
        """Helper function to list options for iscsi

        :param options: command line options
        :type options: list.
        :param iscsipath: current iscsi path
        :type iscsipath: str.
        :param iscsisettingspath: current iscsi settings path
        :type iscsisettingspath: str.
        :param bootpath: current boot path
        :type bootpath: str.
        """
        self.auxcommands["select"].selectfunction("HpeBiosMapping.")
        pcisettingsmap = self.auxcommands["get"].getworkerfunction(
            "BiosPciSettingsMappings", options, results=True, uselist=True
        )

        devicealloc = list()
        for item in pcisettingsmap[0]["BiosPciSettingsMappings"]:
            if "Associations" in item:
                if "EmbNicEnable" in item["Associations"] or "EmbNicConfig" in item["Associations"]:
                    _ = [devicealloc.append(x) for x in item["Subinstances"]]

                if (
                    re.match("FlexLom[0-9]+Enable", item["Associations"][0])
                    or re.match("PciSlot[0-9]+Enable", item["Associations"][0])
                    or re.match("Slot[0-9]+NicBoot[0-9]+", item["Associations"][0])
                ):
                    _ = [devicealloc.append(x) for x in item["Subinstances"]]

        if self.rdmc.app.typepath.defs.isgen10:
            newpcilist = []

            self.auxcommands["select"].selectfunction("HpeServerPciDeviceCollection")
            pcideviceslist = next(
                iter(self.auxcommands["get"].getworkerfunction("Members", options, results=True, uselist=False)),
                None,
            )

            for device in pcideviceslist["Members"]:
                newpcilist.append(self.rdmc.app.get_handler(device["@odata.id"], silent=True).dict)

            pcideviceslist = newpcilist
        else:
            self.auxcommands["select"].selectfunction(["Collection."])
            pcideviceslist = next(
                iter(
                    self.auxcommands["get"].getworkerfunction(
                        "Items",
                        options,
                        results=True,
                        uselist=False,
                        filtervals=("MemberType", "HpServerPciDevice.*"),
                    )
                ),
                None,
            )["Items"]
        i = 0
        iscsibootsources = ""
        while i < 4:
            self.auxcommands["select"].selectfunction("HpiSCSISoftwareInitiator.")
            iscsibootsources = self.rawdatahandler(action="GET", silent=True, jsonflag=True, path=iscsisettingspath)
            if not ("error" in iscsibootsources):
                break
            time.sleep(10)
            i = i + 1
            self.rdmc.ui.printer("Retrying ...\n\n")
        structeredlist = list()

        self.pcidevicehelper(devicealloc, iscsipath, bootpath, pcideviceslist)
        if not ("error" in iscsibootsources):
            for item in iscsibootsources[self.rdmc.app.typepath.defs.iscsisource]:
                if item["iSCSINicSource"]:
                    for device in devicealloc:
                        it = device["CorrelatableID"]
                        it = it.split(",")
                        del it[-1]
                        it = ",".join(it)
                        if not isinstance(device["Associations"][0], list):
                            listval = 1 if isinstance(device["Associations"][0], dict) else 0
                        else:
                            listval = 0
                        if item["iSCSINicSource"] == device["Associations"][listval]:
                            for pcidevice in pcideviceslist:
                                pcid = pcidevice["UEFIDevicePath"]
                                pcid = pcid.split(",")
                                del pcid[-1]
                                pcid = ",".join(pcid)
                                if (device["CorrelatableID"] == pcidevice["UEFIDevicePath"]) or (
                                    "Embedded" not in pcidevice["DeviceType"] and (pcid == it)
                                ):
                                    if "Storage" not in pcidevice["DeviceType"]:
                                        inputstring = (
                                            pcidevice["DeviceType"]
                                            + " "
                                            + str(pcidevice["DeviceInstance"])
                                            + " Port "
                                            + str(device["Subinstance"])
                                            + " : "
                                            + pcidevice.get("Name", "")
                                        )
                                        structeredlist.append(
                                            {
                                                inputstring: {
                                                    str(
                                                        "Attempt "
                                                        + str(item[self.rdmc.app.typepath.defs.iscsiattemptinstance])
                                                    ): item
                                                }
                                            }
                                        )

                else:
                    structeredlist.append({"Not Added": {}})

        try:
            if iscsibootsources is None:
                raise BootOrderMissingEntriesError("No entries found for iscsi boot sources \n\n")
            elif "error" in iscsibootsources:
                raise BootOrderMissingEntriesError(
                    "/redfish/v1/systems/1/bios/oem/hpe/iscsi/settings URI seems to be not reachable even after"
                    " 4 retry attempts , kindly retry the command after some time \n\n"
                )
            elif not options.filename and print_flag:
                self.print_iscsi_config_helper(structeredlist, "Current iSCSI Attempts: \n")
        except Exception as excp:
            raise excp

        if structeredlist is None:
            self.rdmc.ui.error("No entries found for iscsi boot sources.\n\n")
        elif options.filename:
            output = json.dumps(structeredlist, indent=2, cls=redfish.ris.JSONEncoder, sort_keys=True)
            filehndl = open(options.filename[0], "w")
            filehndl.write(output)
            filehndl.close()

            self.rdmc.ui.printer("Results written out to '%s'\n" % options.filename[0])
        return structeredlist

    def defaultsoptionhelper(self, options, iscsipath, bootpath):
        """Helper function for default options for iscsi

        :param options: command line options
        :type options: list.
        :param iscsipath: current iscsi path
        :type iscsipath: str.
        :param bootpath: current boot path
        :type bootpath: str.
        """
        self.auxcommands["select"].selectfunction("HpBiosMapping.")
        pcisettingsmap = self.auxcommands["get"].getworkerfunction(
            "BiosPciSettingsMappings", options, results=True, uselist=True
        )

        devicealloc = list()
        for item in pcisettingsmap[0]["BiosPciSettingsMappings"]:
            if "Associations" in item:
                # self.rdmc.ui.printer("Assoc 1 : %s\n" % (item["Associations"]))
                # self.rdmc.ui.printer("Sub 1 : %s\n" % (item["Subinstances"]))
                if "EmbNicEnable" in item["Associations"] or "EmbNicConfig" in item["Associations"]:
                    _ = [devicealloc.append(x) for x in item["Subinstances"]]

                try:
                    item["Associations"][0]
                    # self.rdmc.ui.printer("Assoc 2 : %s\n" % (item["Associations"][0]))
                    if (
                        re.match("FlexLom[0-9]+Enable", item["Associations"][0])
                        or re.match("PciSlot[0-9]+Enable", item["Associations"][0])
                        or re.match("Slot[0-9]+NicBoot[0-9]+", item["Associations"][0])
                    ):
                        _ = [devicealloc.append(x) for x in item["Subinstances"]]
                except IndexError:
                    pass
            # self.rdmc.ui.printer("Device alloc : %s\n" % devicealloc)

        if self.rdmc.app.typepath.defs.isgen10:
            newpcilist = []
            self.auxcommands["select"].selectfunction("HpeServerPciDeviceCollection")
            pcideviceslist = self.auxcommands["get"].getworkerfunction("Members", options, results=True, uselist=False)
            pcideviceslist = pcideviceslist[0]
            for device in pcideviceslist["Members"]:
                newpcilist.append(self.rdmc.app.get_handler(device["@odata.id"], silent=True).dict)
            pcideviceslist = newpcilist
        else:
            self.auxcommands["select"].selectfunction(["Collection."])
            pcideviceslist = self.auxcommands["get"].getworkerfunction(
                "Items",
                options,
                results=True,
                uselist=False,
                filtervals=("MemberType", "HpServerPciDevice.*"),
            )[0]["Items"]

        self.auxcommands["select"].selectfunction(self.rdmc.app.typepath.defs.hpiscsisoftwareinitiatortype)
        iscsiinitiatorname = self.auxcommands["get"].getworkerfunction(
            "iSCSIInitiatorName", options, results=True, uselist=True
        )

        disabledlist = self.pcidevicehelper(devicealloc, iscsipath, bootpath, pcideviceslist)

        self.print_out_iscsi_configuration(iscsiinitiatorname[0], devicealloc, pcideviceslist)

        if disabledlist:
            self.print_out_iscsi_configuration(iscsiinitiatorname[0], disabledlist, pcideviceslist, disabled=True)

    def modifyoptionhelper(self, options, iscsisettingspath):
        """Helper function to modify options for iscsi

        :param options: command line options
        :type options: list.
        :param iscsisettingspath: current iscsi settings path
        :type iscsisettingspath: str.
        """
        try:
            inputfile = open(options.modify, "r")
            contentsholder = json.loads(inputfile.read())
        except Exception as excp:
            raise InvalidCommandLineError("%s" % excp)

        iscsibootsources = self.rawdatahandler(action="GET", silent=True, jsonflag=True, path=iscsisettingspath)

        count = 0
        resultsdict = list()

        for item in contentsholder:
            for entry in item.values():
                enteredsection = False

                for key, value in entry.items():
                    enteredsection = True
                    resultsdict.append(
                        self.modifyfunctionhelper(
                            key,
                            value,
                            iscsibootsources[self.rdmc.app.typepath.defs.iscsisource],
                        )
                    )

                if not enteredsection:
                    resultsdict.append(iscsibootsources[self.rdmc.app.typepath.defs.iscsisource][count])

                count += 1

        contentsholder = {self.rdmc.app.typepath.defs.iscsisource: resultsdict}

        self.rdmc.app.patch_handler(iscsisettingspath, contentsholder, optionalpassword=options.biospassword)
        self.rdmc.app.get_handler(iscsisettingspath, silent=True)
        self.rdmc.ui.printer("Please reboot the server for changes to take effect\n")

    def modifyfunctionhelper(self, key, value, bootsources):
        """Helper function to modify the entries for iscsi

        :param key: key to be used for attempt
        :type key: string.
        :param value: value to apply to attempt
        :type value: str.
        :param bootsources: current boot sources
        :type bootsources: list.
        """
        foundoption = False

        for bootsource in bootsources:
            if bootsource[self.rdmc.app.typepath.defs.iscsiattemptinstance] == int(key[-1:]):
                foundoption = True
                break
            else:
                return value
        if foundoption:
            return value

    def pcidevicehelper(self, devicealloc, iscsipath, bootpath, pcideviceslist=None, options=None):
        """Helper function to check for extra pci devices / identify disabled devices

        :param devicealloc: list of devices allocated
        :type devicealloc: list.
        :param iscsipath: current iscsi path
        :type iscsipath: str.
        :param bootpath: current boot path
        :type bootpath: str.
        :param pcideviceslist: current pci device list
        :type pcideviceslist: list.
        :param options: command line options
        :type options: list.
        """
        if not pcideviceslist:
            if self.rdmc.app.typepath.defs.isgen10:
                newpcilist = []
                self.auxcommands["select"].selectfunction("HpeServerPciDeviceCollection")
                pcideviceslist = next(
                    iter(self.auxcommands["get"].getworkerfunction("Members", options, results=True, uselist=False)),
                    None,
                )

                for device in pcideviceslist["Members"]:
                    newpcilist.append(self.rdmc.app.get_handler(device["@odata.id"], silent=True).dict)

                pcideviceslist = newpcilist
            else:
                self.auxcommands["select"].selectfunction(["Collection."])
                pcideviceslist = next(
                    iter(
                        self.auxcommands["get"].getworkerfunction(
                            "Items",
                            options,
                            results=True,
                            uselist=False,
                            filtervals=("MemberType", "HpServerPciDevice.*"),
                        )
                    ),
                    None,
                )["Items"]
        try:
            self.rawdatahandler(action="GET", silent=True, jsonflag=True, path=iscsipath)["iSCSINicSources"]
        except:
            raise NicMissingOrConfigurationError("No iSCSI nic sources available.")

        _ = [x["UEFIDevicePath"] for x in pcideviceslist]
        removal = list()

        bios = self.rawdatahandler(action="GET", silent=True, jsonflag=True, path=bootpath)

        for item in devicealloc:
            if isinstance(item["Associations"][0], dict):
                if "PreBootNetwork" in list(item["Associations"][0].keys()):
                    if item["Associations"] and item["Associations"][0]["PreBootNetwork"] in list(bios.keys()):
                        if bios[item["Associations"][0]["PreBootNetwork"]] == "Disabled":
                            removal.append(item)
            else:
                if item["Associations"] and item["Associations"][0] in list(bios.keys()):
                    if bios[item["Associations"][0]] == "Disabled":
                        removal.append(item)

        _ = [devicealloc.remove(x) for x in removal]

        return removal

    def print_out_iscsi_configuration(self, iscsiinitiatorname, devicealloc, pcideviceslist, disabled=False):
        """Convert content to human readable and print out to std.out

        :param iscsiinitiatorname: iscsi initiator name
        :type iscsiinitiatorname: str.
        :param devicealloc: list of devices allocated
        :type devicealloc: list.
        :param pcideviceslist: current pci device list
        :type pcideviceslist: list.
        :param disabled: command line options
        :type disabled: boolean.
        """
        try:
            if iscsiinitiatorname is None:
                BootOrderMissingEntriesError("No entry found for the iscsi initiator name.\n\n")
            elif disabled:
                pass
            else:
                self.print_iscsi_config_helper(iscsiinitiatorname["iSCSIInitiatorName"], "\nIscsi Initiator Name: ")
        except Exception as excp:
            raise excp

        try:
            tot_instance = []
            final_device_list = list()
            enable_disable = dict()
            enable_val = list()
            import re

            bios = "/redfish/v1/systems/1/bios/"
            bios_dict = self.rdmc.app.get_handler(bios, silent=True, service=True).dict
            if "Attributes" in bios_dict:
                bios_dict_items = bios_dict["Attributes"].items()
            else:
                bios_dict_items = bios_dict.items()
            for attr, val in bios_dict_items:
                if "NetworkBoot" in str(val):
                    nics = re.findall("\\d+", attr)
                    enable_disable[attr] = nics
                    enable_val.append(nics)
            if devicealloc and pcideviceslist:
                if disabled:
                    self.rdmc.ui.printer("\nDisabled iSCSI Boot Network Interfaces: \n")
                    count = "Disabled"
                else:
                    self.rdmc.ui.printer("Available iSCSI Boot Network Interfaces: \n")
                    count = 1
                for pcidevice in pcideviceslist:
                    pcid = pcidevice["UEFIDevicePath"]
                    pcid = pcid.split(",")
                    del pcid[-1]
                    pcid = ",".join(pcid)
                    for item in devicealloc:
                        it = item["CorrelatableID"]
                        it = it.split(",")
                        del it[-1]
                        it = ",".join(it)
                        if pcid == it:
                            if "Storage" not in pcidevice["DeviceType"]:
                                device = (
                                    pcidevice["DeviceType"]
                                    + " "
                                    + str(pcidevice["DeviceInstance"])
                                    + " Port "
                                    + str(item["Subinstance"])
                                )
                                if device not in final_device_list:
                                    final_device_list.append(device)
                                    temp = dict()
                                    temp["Device"] = device
                                    if "Name" not in pcidevice:
                                        temp["Name"] = pcidevice["StructuredName"]
                                    else:
                                        temp["Name"] = pcidevice["Name"]
                                    tot_instance.append(temp)
                count = 0
                enable_nic = dict()
                # enb_val = list()
                for t in tot_instance:
                    nic = t["Device"]
                    nics = re.findall("\\d+", nic)
                    enable_nic[nic] = nics
                    if "Embedded LOM" in t["Device"]:
                        nics.remove("1")
                    for i in enable_val:
                        if i == nics:
                            count = count + 1
                            self.rdmc.ui.printer(
                                "[%s] %s : %s\n"
                                % (
                                    count,
                                    t["Device"],
                                    t["Name"],
                                )
                            )
            else:
                raise BootOrderMissingEntriesError("No entries found for" " iscsi configurations devices.\n")
        except Exception as excp:
            raise excp

    def print_iscsi_config_helper(self, content, outstring, indent=0):
        """Print iscsi configuration helper

        :param content: current content to be output
        :type content: string.
        :param outstring: current output string
        :type outstring: str.
        :param indent: current iscsi settings path
        :type indent: str.
        """
        self.rdmc.ui.printer("\t" * indent + outstring)

        if content:
            self.rdmc.ui.print_out_json(content)
        else:
            self.rdmc.ui.error("\t" * indent + "No entries currently configured.\n")

        self.rdmc.ui.printer("\n\n")

    def rawdatahandler(self, action="None", path=None, silent=True, jsonflag=False):
        """Helper function to get and put the raw data

        :param action: current rest action
        :type action: list.
        :param path: current path
        :type path: str.
        :param silent: flag to determine silent mode
        :type silent: boolean.
        :param jsonflag: flag to determine json output
        :type jsonflag: boolean.
        """
        if action == "GET":
            rawdata = self.rdmc.app.get_handler(get_path=path, silent=silent)

        if jsonflag is True:
            rawdata = json.loads(rawdata.read)

        return rawdata

    def validateinput(self, deviceallocsize=None, options=None, deleteoption=False):
        """Helper function to validate that the input is correct

        :param deviceallocsize: current device allocated size
        :type deviceallocsize: str.
        :param options: command line options
        :type options: list.
        :param deleteoption: flag to delete option
        :type deleteoption: boolean.
        """
        if deviceallocsize:
            try:
                if int(options.add) - 1 >= deviceallocsize:
                    raise NicMissingOrConfigurationError("Please verify the " "given input value for configuring NIC.")
            except Exception:
                raise NicMissingOrConfigurationError("Please verify the " "given input value for configuring NIC.")
        if deleteoption:
            try:
                if int(options.delete) == 0:
                    raise NicMissingOrConfigurationError(
                        "Invalid input value." "Please give valid attempt for instance values."
                    )
            except Exception:
                raise NicMissingOrConfigurationError(
                    "Invalid input value." "Please give valid attempt for instance values."
                )

    def iscsiconfigurationvalidation(self, options):
        """iscsi configuration method validation function

        :param options: command line options
        :type options: list.
        """
        self.cmdbase.login_select_validation(self, options)

        if options.encode:
            options.biospassword = Encryption.decode_credentials(options.biospassword)
            if isinstance(options.biospassword, bytes):
                options.biospassword = options.biospassword.decode("utf-8")

    def definearguments(self, customparser):
        """Wrapper function for new command main function

        :param customparser: command line input
        :type customparser: parser.
        """
        if not customparser:
            return

        self.cmdbase.add_login_arguments_group(customparser)

        customparser.add_argument(
            "-f",
            "--filename",
            dest="filename",
            help="Use this flag if you wish to use a different"
            " filename than the default one. The default filename is"
            " ilorest.json.",
            action="append",
            default=None,
        )
        customparser.add_argument(
            "--add",
            dest="add",
            help="Use this iSCSI configuration option to add an iSCSI" " configuration option.",
            default=None,
        )
        customparser.add_argument(
            "--delete",
            dest="delete",
            help="Use this iSCSI configuration option to delete an iSCSI" " configuration option.",
            default=None,
        )
        customparser.add_argument(
            "--modify",
            dest="modify",
            help="Use this iSCSI configuration option to modify an iSCSI" " configuration option.",
            default=None,
        )
        customparser.add_argument(
            "--list",
            dest="list",
            action="store_true",
            help="Use this iSCSI configuration option to list the details" " of the different iSCSI configurations.",
            default=None,
        )
        customparser.add_argument(
            "--reboot",
            dest="reboot",
            help="Use this flag to perform a reboot command function after"
            " completion of operations.  For help with parameters and"
            " descriptions regarding the reboot flag, run help reboot.",
            default=None,
        )



from setuptools import find_packages, setup
import site

site_packages = site.getsitepackages()

extras = {}

setup(name='ilorest',
      version='5.3.0.0',
      description='HPE iLORest Tool',
      author='Hewlett Packard Enterprise',
      author_email='rajeevalochana.kallur@hpe.com',
      extras_require=extras,
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'License :: OSI Approved :: Apache Software License',
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: 3.8',
          'Programming Language :: Python :: 3.9',
          'Programming Language :: Python :: 3.10',
          'Programming Language :: Python :: 3.11',
          'Programming Language :: Python :: 3.12',
          'Topic :: Communications'
      ],
      keywords='Hewlett Packard Enterprise',
      url='https://github.com/HewlettPackard/python-redfish-utility',
      packages=find_packages('.', exclude=["tests", "docs"]),
      package_dir={'': '.'},
      package_data={"ilorest.chiflibrary": ["ilorest_chif.dll", "ilorest_chif.so"]},
      entry_points={
        'console_scripts': [
          'ilorest = ilorest.rdmc:ilorestcommand',
        ],
      },
      install_requires=[
          'urllib3 >= 1.26.2',
          'pyaes >= 1.6.1',
          'colorama >= 0.4.4',
          'jsonpointer >= 2.0',
          'six >= 1.15.0',
          'ply',
          'decorator >= 4.4.2',
          'jsonpatch >= 1.28',
          'jsonpath-rw >= 1.4.0',
          'setproctitle >= 1.1.8; platform_system == "Linux"',
          'jsondiff >= 1.2.0',
          'tabulate >= 0.8.7',
          'prompt_toolkit',
          'certifi >= 2020.12.5',
          'pywin32; platform_system == "Windows"',
          'wcwidth >= 0.2.5',
          'pyudev',
          'future',
          'enum; python_version <= "2.7.19"',
          'futures; python_version <= "2.7.19"',
          'python-ilorest-library >= 5.3.0.0',
      ])



###
# Copyright 2016-2023 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

# -*- coding: utf-8 -*-
""" Set Command for RDMC """
import json
import os
import sys

import redfish.ris

try:
    from rdmc_helper import (
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        InvalidOrNothingChangedSettingsError,
        ReturnCodes,
    )
except ImportError:
    from ilorest.rdmc_helper import (
        ReturnCodes,
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        InvalidOrNothingChangedSettingsError,
    )

from redfish.ris.rmc_helper import NothingSelectedError


class SetCommand:
    """Constructor"""

    def __init__(self):
        self.ident = {
            "name": "set",
            "usage": None,
            "description": "Setting a "
            "single level property example:\n\tset property=value\n\n\t"
            "Setting multiple single level properties example:\n\tset "
            "property=value property=value property=value\n\n\t"
            "Setting a multi level property example:\n\tset property/"
            "subproperty=value",
            "summary": "Changes the value of a property within the" " currently selected type.",
            "aliases": [],
            "auxcommands": [
                "CommitCommand",
                "RebootCommand",
                "RawPatchCommand",
                "SelectCommand",
                "LogoutCommand",
            ],
        }
        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()

    def setfunction(self, line, skipprint=False):
        """Main set worker function

        :param line: command line input
        :type line: string.
        :param skipprint: boolean to determine output
        :type skipprint: boolean.
        """

        try:
            (options, args) = self.rdmc.rdmc_parse_arglist(self, line)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineErrorOPTS("")

        if not self.rdmc.interactive and not self.rdmc.app.cache:
            raise InvalidCommandLineError("The 'set' command is not useful in " "non-interactive and non-cache modes.")

        self.setvalidation(options)
        _ = args[0].find("HighSecurity")

        fsel = None
        fval = None
        if args:
            if "ethernetinterface" in self.rdmc.app.selector.lower() and "ipv4addresses" in args[0].lower():
                raise InvalidCommandLineError(
                    "For security reasons this request is discarded. "
                    "Use `rawpatch` or the `load --force_network_config` command "
                    "to modify the IP configuration of the ilO.\n"
                )
            if options.filter:
                try:
                    (fsel, fval) = str(options.filter).strip("'\" ").split("=")
                    (fsel, fval) = (fsel.strip(), fval.strip())
                except:
                    raise InvalidCommandLineError(
                        "Invalid filter" " parameter format [filter_attribute]=[filter_value]"
                    )

            if any([s.lower().startswith("adminpassword=") for s in args]) and not any(
                [s.lower().startswith("oldadminpassword=") for s in args]
            ):
                raise InvalidCommandLineError(
                    "'OldAdminPassword' must also "
                    "be set with the current password \nwhen "
                    "changing 'AdminPassword' for security reasons."
                )
            count = 0
            for arg in args:
                if arg:
                    if len(arg) > 2:
                        if ('"' in arg[0] and '"' in arg[-1]) or ("'" in arg[0] and "'" in arg[-1]):
                            args[count] = arg[1:-1]
                    elif len(arg) == 2:
                        if (arg[0] == '"' and arg[1] == '"') or (arg[0] == "'" and arg[1] == "'"):
                            args[count] = None
                count += 1
                if not self.rdmc.app.selector:
                    raise NothingSelectedError
                if "." not in self.rdmc.app.selector:
                    self.rdmc.app.selector = self.rdmc.app.selector + "."
                if self.rdmc.app.selector:
                    if self.rdmc.app.selector.lower().startswith("bios."):
                        if "attributes" not in arg.lower():
                            arg = "Attributes/" + arg

                try:
                    (sel, val) = arg.split("=")
                    sel = sel.strip().lower()
                    val = val.strip("\"'")

                    if val.lower() == "true" or val.lower() == "false":
                        val = val.lower() in ("yes", "true", "t", "1")
                except:
                    raise InvalidCommandLineError("Invalid set parameter format. [Key]=[Value]")

                newargs = list()

                if "/" in sel and "/" not in str(val):
                    newargs = sel.split("/")
                elif "/" in sel:
                    items = arg.split("=", 1)
                    newargs = items[0].split("/")
                if not isinstance(val, bool):
                    if val:
                        if val[0] == "[" and val[-1] == "]":
                            val = val[1:-1].split(",")
                payload = {newargs[-1]: val} if newargs else {sel: val}
                if newargs:
                    for key in newargs[:-1][::-1]:
                        payload = {key: payload}
                try:
                    contents = self.rdmc.app.loadset(
                        seldict=payload,
                        latestschema=options.latestschema,
                        fltrvals=(fsel, fval),
                        uniqueoverride=options.uniqueoverride,
                    )
                    if not contents:
                        if not sel.lower() == "oldadminpassword":
                            raise InvalidOrNothingChangedSettingsError(
                                "Nothing changed "
                                "for attribute '%s'.\nPlease check if the attribute is Oem/Hpe Attribute or Read-only "
                                "or System Unique property or the value trying to set is same or invalid" % sel
                            )
                    elif contents == "No entries found":
                        raise InvalidOrNothingChangedSettingsError(
                            "No " "entries found in the current " "selection for the setting '%s'." % sel
                        )
                    elif contents == "reverting":
                        self.rdmc.ui.error("Removing previous patch and returning to the " "original value.\n")
                    else:
                        for content in contents:
                            self.rdmc.ui.printer("Added the following patch:\n")
                            self.rdmc.ui.print_out_json(content)

                except redfish.ris.ValidationError as excp:
                    errs = excp.get_errors()

                    for err in errs:
                        if err.sel and err.sel.lower() == "adminpassword":
                            types = self.rdmc.app.monolith.types

                            for item in types:
                                for instance in types[item]["Instances"]:
                                    if "hpbios." in instance.maj_type.lower():
                                        _ = [
                                            instance.patches.remove(patch)
                                            for patch in instance.patches
                                            if patch.patch[0]["path"] == "/OldAdminPassword"
                                        ]

                        if isinstance(err, redfish.ris.RegistryValidationError):
                            self.rdmc.ui.printer(err.message)

                    raise redfish.ris.ValidationError(excp)

            if options.commit:
                self.auxcommands["commit"].commitfunction(options)

            if options.reboot and not options.commit:
                self.auxcommands["reboot"].run(options.reboot)

            if options.logout:
                self.auxcommands["logout"].run("")

        else:
            raise InvalidCommandLineError("Missing parameters for 'set' command.\n")

    def patchfunction(self, line):
        """Main set worker function
        :param line: command line input
        :type line: string.
        :param skipprint: boolean to determine output
        :type skipprint: boolean.
        """
        try:
            (options, args) = self.rdmc.rdmc_parse_arglist(self, line)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineErrorOPTS("")

        if not self.rdmc.interactive and not self.rdmc.app.cache:
            raise InvalidCommandLineError("The 'set' command is not useful in " "non-interactive and non-cache modes.")

        _ = self.rdmc.app.selector
        self.setvalidation(options)
        if args:
            count = 0
            for arg in args:
                if arg:
                    if len(arg) > 2:
                        if ('"' in arg[0] and '"' in arg[-1]) or ("'" in arg[0] and "'" in arg[-1]):
                            args[count] = arg[1:-1]
                    elif len(arg) == 2:
                        if (arg[0] == '"' and arg[1] == '"') or (arg[0] == "'" and arg[1] == "'"):
                            args[count] = None
                count += 1
                if not self.rdmc.app.selector:
                    raise NothingSelectedError
                if "." not in self.rdmc.app.selector:
                    self.rdmc.app.selector = self.rdmc.app.selector + "."
                if self.rdmc.app.selector:
                    if self.rdmc.app.selector.lower().startswith("bios."):
                        if "attributes" not in arg.lower():
                            arg = "Attributes/" + arg

                try:
                    (sel, val) = arg.split("=")
                    sel = sel.strip()
                    val = val.strip("\"'")

                    if val.lower() == "true" or val.lower() == "false":
                        val = val.lower() in ("yes", "true", "t", "1")
                except:
                    raise InvalidCommandLineError("Invalid set parameter format. [Key]=[Value]")

                newargs = list()

                if "/" in sel and "/" not in str(val):
                    newargs = sel.split("/")
                elif "/" in sel:
                    items = arg.split("=", 1)
                    newargs = items[0].split("/")

                if not isinstance(val, bool):
                    if val:
                        if val[0] == "[" and val[-1] == "]":
                            val = val[1:-1].split(",")
                    if val.isdigit():
                        val = int(val)
                patch_data = dict()
                payload = {newargs[-1]: val} if newargs else {sel: val}
                if newargs:
                    for key in newargs[:-1][::-1]:
                        if key == "PowerControl":
                            payload = {key: [payload]}
                        else:
                            payload = {key: payload}
                        patch_data.update(payload)
                        if "Oem" in key:
                            if (
                                "managernetworkprotocol." in self.rdmc.app.selector
                                or "thermal." in self.rdmc.app.selector
                            ):
                                import platform
                                import tempfile

                                tempdir = "/tmp" if platform.system() == "Darwin" else tempfile.gettempdir()
                                temp_file = os.path.join(tempdir, "temp_patch.json")
                                out_file = open(temp_file, "w")
                                if "managernetworkprotocol." in self.rdmc.app.selector:
                                    patchpath = self.rdmc.app.typepath.defs.managerpath + "NetworkProtocol/"
                                elif "thermal." in self.rdmc.app.selector:
                                    patchpath = "/redfish/v1/Chassis/1/Thermal/"
                                patch_payload = {patchpath: payload}
                                json.dump(patch_payload, out_file, indent=6)
                                out_file.close()
                                sys.stdout.write("payload %s \n" % patch_payload)
                                self.auxcommands["rawpatch"].run(temp_file + " --service")
                                self.auxcommands["select"].run(self.rdmc.app.selector + " --refresh")
                                os.remove(temp_file)
                    if "PowerControl" in payload.keys():
                        import platform
                        import tempfile

                        tempdir = "/tmp" if platform.system() == "Darwin" else tempfile.gettempdir()
                        temp_file = os.path.join(tempdir, "temp_patch.json")
                        out_file = open(temp_file, "w")
                        patch_path = "/redfish/v1/Chassis/1/Power"
                        patch_payload = {patch_path: payload}
                        json.dump(patch_payload, out_file, indent=6)
                        out_file.close()
                        self.auxcommands["rawpatch"].run(temp_file + " --service")
                        os.remove(temp_file)

    def run(self, line, skipprint=False, help_disp=False):
        """Main set function

        :param line: command line input
        :type line: string.
        :param skipprint: boolean to determine output
        :type skipprint: boolean.
        """
        if help_disp:
            self.parser.print_help()
            return ReturnCodes.SUCCESS
        if (
            ("Oem/Hpe/EnhancedDownloadPerformanceEnabled" in line[0])
            or ("Oem/Hpe/ThermalConfiguration" in line[0])
            or ("Oem/Hpe/FanPercentMinimum" in line[0])
            or ("PowerControl" in line[0])
            or ("drive" in self.rdmc.app.selector.lower())
        ):
            self.patchfunction(line)
        else:
            self.setfunction(line, skipprint=skipprint)
        # Return code
        return ReturnCodes.SUCCESS

    def setvalidation(self, options):
        """Set data validation function"""

        if self.rdmc.opts.latestschema:
            options.latestschema = True
        if self.rdmc.config.commit.lower() == "true":
            options.commit = True
        try:
            self.cmdbase.login_select_validation(self, options)
        except redfish.ris.NothingSelectedError:
            raise redfish.ris.NothingSelectedSetError("")

    def definearguments(self, customparser):
        """Wrapper function for new command main function

        :param customparser: command line input
        :type customparser: parser.
        """
        if not customparser:
            return

        self.cmdbase.add_login_arguments_group(customparser)

        customparser.add_argument(
            "--selector",
            dest="selector",
            help="Optionally include this flag to select a type to run"
            " the current command on. Use this flag when you wish to"
            " select a type without entering another command, or if you"
            " wish to work with a type that is different from the one"
            " you currently have selected.",
            default=None,
        )

        customparser.add_argument(
            "--filter",
            dest="filter",
            help="Optionally set a filter value for a filter attribute."
            " This uses the provided filter for the currently selected"
            " type. Note: Use this flag to narrow down your results. For"
            " example, selecting a common type might return multiple"
            " objects that are all of that type. If you want to modify"
            " the properties of only one of those objects, use the filter"
            " flag to narrow down results based on properties."
            "\t\t\t\t\t Usage: --filter [ATTRIBUTE]=[VALUE]",
            default=None,
        )
        customparser.add_argument(
            "--commit",
            dest="commit",
            action="store_true",
            help="Use this flag when you are ready to commit all pending"
            " changes. Note that some changes made in this way will be updated"
            " instantly, while others will be reflected the next time the"
            " server is started.",
            default=None,
        )
        customparser.add_argument(
            "--reboot",
            dest="reboot",
            help="Use this flag to perform a reboot command function after"
            " completion of operations.  For help with parameters and"
            " descriptions regarding the reboot flag, run help reboot.",
            default=None,
        )
        customparser.add_argument(
            "--latestschema",
            dest="latestschema",
            action="store_true",
            help="Optionally use the latest schema instead of the one "
            "requested by the file. Note: May cause errors in some data "
            "retrieval due to difference in schema versions.",
            default=None,
        )
        customparser.add_argument(
            "--uniqueoverride",
            dest="uniqueoverride",
            action="store_true",
            help="Override the measures stopping the tool from writing " "over items that are system unique.",
            default=None,
        )



###
# Copyright 2016-2021 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

# -*- coding: utf-8 -*-
""" iLO Reset Command for rdmc """

from redfish.ris.rmc_helper import IloResponseError

try:
    from rdmc_helper import (
        InvalidCommandLineErrorOPTS,
        NoContentsFoundForOperationError,
        ReturnCodes,
    )
except ImportError:
    from ilorest.rdmc_helper import (
        InvalidCommandLineErrorOPTS,
        NoContentsFoundForOperationError,
        ReturnCodes,
    )


class IloResetCommand:
    """Reset iLO on the server that is currently logged in"""

    def __init__(self):
        self.ident = {
            "name": "iloreset",
            "usage": None,
            "description": "Reset iLO on the current logged in" " server.\n\tExample: iloreset",
            "summary": "Reset iLO on the current logged in server.",
            "aliases": [],
            "auxcommands": [],
        }

        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()

    def run(self, line, help_disp=False):
        """Main iLO reset worker function

        :param line: string of arguments passed in
        :type line: str.
        :param help_disp: display help flag
        :type line: bool.
        """
        if help_disp:
            self.parser.print_help()
            return ReturnCodes.SUCCESS
        try:
            (options, _) = self.rdmc.rdmc_parse_arglist(self, line)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineErrorOPTS("")

        self.iloresetvalidation(options)

        self.rdmc.ui.warn(
            "\nAfter iLO resets, the session will be terminated."
            "\nPlease wait for iLO to initialize completely before logging "
            "in again.\nThis process may take up to 3 minutes to complete.\n\n"
        )

        select = "/redfish/v1/Managers/1/"
        results = self.rdmc.app.get_handler(select, silent=True, service=True).dict
        try:
            results = results
        except:
            pass

        if results:
            post_path = results["Actions"]["#Manager.Reset"]["target"]
        else:
            raise NoContentsFoundForOperationError("Unable to find %s" % select)

        bodydict = results

        try:
            for item in bodydict["Actions"]:
                if "Reset" in item:
                    if self.rdmc.app.typepath.defs.isgen10:
                        action = item.split("#")[-1]
                    else:
                        action = "Reset"

                    post_path = bodydict["Actions"][item]["target"]
                    break
        except:
            action = "Reset"

        body = {"Action": action}

        postres = self.rdmc.app.post_handler(post_path, body, silent=True, service=True)
        if postres.status == 200:
            self.rdmc.ui.printer("A management processor reset is in progress.\n")
        else:
            self.rdmc.ui.error("An error occured during iLO reset.\n")
            raise IloResponseError("")

        self.cmdbase.logout_routine(self, options)
        # Return code
        return ReturnCodes.SUCCESS

    def iloresetvalidation(self, options):
        """reboot method validation function

        :param options: command line options
        :type options: list.
        """
        self.cmdbase.login_select_validation(self, options)

    def definearguments(self, customparser):
        """Wrapper function for new command main function

        :param customparser: command line input
        :type customparser: parser.
        """
        if not customparser:
            return

        self.cmdbase.add_login_arguments_group(customparser)



###
# Copyright 2016-2021 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

# -*- coding: utf-8 -*-
""" Add Federation Command for rdmc """


from argparse import Action, RawDescriptionHelpFormatter

from redfish.ris.ris import SessionExpired
from redfish.ris.rmc_helper import IdTokenError

try:
    from rdmc_helper import (
        IncompatibleiLOVersionError,
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        NoContentsFoundForOperationError,
        ResourceExists,
        ReturnCodes,
        UsernamePasswordRequiredError,
    )
except ImportError:
    from ilorest.rdmc_helper import (
        IncompatibleiLOVersionError,
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        NoContentsFoundForOperationError,
        ResourceExists,
        ReturnCodes,
        UsernamePasswordRequiredError,
    )

__subparsers__ = ["add", "modify", "changekey", "delete"]


class _FederationParse(Action):
    def __init__(self, option_strings, dest, nargs, **kwargs):
        super(_FederationParse, self).__init__(option_strings, dest, nargs, **kwargs)

    def __call__(self, parser, namespace, values, option_strings):
        """Federation privileges option helper"""

        privkey = {
            1: "LoginPriv",
            2: "RemoteConsolePriv",
            3: "UserConfigPriv",
            4: "iLOConfigPriv",
            5: "VirtualMediaPriv",
            6: "VirtualPowerAndResetPriv",
            7: "HostNICConfigPriv",
            8: "HostBIOSConfigPriv",
            9: "HostStorageConfigPriv",
            10: "SystemRecoveryConfigPriv",
        }

        for priv in next(iter(values)).split(","):
            try:
                priv = int(priv)
            except ValueError:
                try:
                    parser.error("Invalid privilege entered: %s. Privileges must " "be numbers." % priv)
                except:
                    raise InvalidCommandLineErrorOPTS("")

            try:
                if not isinstance(namespace.optprivs, list):
                    namespace.optprivs = list()
                if option_strings.startswith("--add"):
                    namespace.optprivs.append({privkey[priv]: True})
                elif option_strings.startswith("--remove"):
                    namespace.optprivs.append({privkey[priv]: False})
            except KeyError:
                try:
                    parser.error(
                        "Invalid privilege entered: %s. Number does not " "match an available privlege." % priv
                    )
                except:
                    raise InvalidCommandLineErrorOPTS("")


class IloFederationCommand:
    """Add a new ilo federation to the server"""

    def __init__(self):
        self.ident = {
            "name": "ilofederation",
            "usage": None,
            "description": "View, Add, Remove and Modify iLO Federation accoutns based on the "
            "sub-command used.\nTo view help on specific sub-commands run: ilofederation "
            "<sub-command> -h\n\nExample: ilofederation add -h\n"
            "NOTE 1: By default only the login privilege is added to the newly created\n\t\t"
            'federation group with role "ReadOnly" in iLO 5 and no privileges in iLO 4.\n\t'
            "NOTE 2: Federation credentials are case-sensitive.",
            "summary": "Adds / deletes an iLO federation group on the currently logged in server.",
            "aliases": [],
            "auxcommands": [],
        }

        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()

    def run(self, line, help_disp=False):
        """Main addfederation function
        :param line: string of arguments passed in
        :type line: str.
        :param help_disp: display help flag
        :type line: bool.
        """
        if help_disp:
            line.append("-h")
            try:
                (_, _) = self.rdmc.rdmc_parse_arglist(self, line)
            except:
                return ReturnCodes.SUCCESS
            return ReturnCodes.SUCCESS
        body = dict()
        try:
            ident_subparser = False
            for cmnd in __subparsers__:
                if cmnd in line:
                    (options, args) = self.rdmc.rdmc_parse_arglist(self, line)
                    ident_subparser = True
                    break
            if not ident_subparser:
                (options, args) = self.rdmc.rdmc_parse_arglist(self, line, default=True)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineErrorOPTS("")

        self.ilofederationvalidation(options)

        redfish = self.rdmc.app.monolith.is_redfish
        path = self.rdmc.app.typepath.defs.federationpath
        results = self.rdmc.app.get_handler(path, service=True, silent=True).dict

        if redfish:
            results = results["Members"]
        else:
            if "Member" in results["links"]:
                results = results["links"]["Member"]
            else:
                results = list()

        mod_fed = None
        path = None
        for indx, acct in enumerate(results):
            fed = self.rdmc.app.get_handler(
                acct[self.rdmc.app.typepath.defs.hrefstring], service=True, silent=True
            ).dict
            try:
                if hasattr(options, "fedname"):
                    if fed["Name"].lower() == options.fedname.lower():
                        if redfish:
                            path = acct["@odata.id"]
                        else:
                            path = acct["href"]
                        mod_fed = fed
                elif options.command == "default":
                    results[indx] = fed
                else:
                    raise KeyError
            except KeyError:
                continue
            else:
                if mod_fed:
                    break

        if mod_fed:
            if options.command.lower() == "add":
                raise ResourceExists("Federation name %s is already in use." % options.fedname)
        else:
            if options.command.lower() != "add" and options.command.lower() != "default":
                raise InvalidCommandLineError("Unable to find the specified federation %s." % options.fedname)

        if options.command.lower() == "add":
            privs = self.getprivs(options)
            path = self.rdmc.app.typepath.defs.federationpath

            body = {"Name": options.fedname, "Key": options.fedkey}
            if privs:
                body.update({"Privileges": privs})
            self.addvalidation(options.fedname, options.fedkey, results)

            if path and body:
                resp = self.rdmc.app.post_handler(path, body, silent=False, service=False)

            if resp and resp.dict:
                if "resourcealreadyexist" in str(resp.dict).lower():
                    raise ResourceExists("")

        elif options.command.lower() == "changekey":
            try:
                newkey = options.fedkey
            except:
                raise InvalidCommandLineError("Invalid number of parameters.")

            body = {"Key": newkey}

            if path and body:
                self.rdmc.app.patch_handler(path, body, service=True)
            else:
                raise NoContentsFoundForOperationError("Unable to find " "the specified federation.")
        elif options.command.lower() == "modify":
            if options.optprivs:
                body.update({"Privileges": {}})
                if any(
                    priv for priv in options.optprivs if "SystemRecoveryConfigPriv" in priv
                ) and "SystemRecoveryConfigPriv" not in list(self.getsesprivs().keys()):
                    raise IdTokenError(
                        "The currently logged in federation must have The System "
                        "Recovery Config privilege to add the System Recovery "
                        "Config privilege."
                    )
                privs = self.getprivs(options)
                body["Privileges"] = privs

            self.rdmc.app.patch_handler(path, body)

        elif options.command.lower() == "delete":
            if path:
                self.rdmc.app.delete_handler(path)
        else:
            if len(results) == 0:
                self.rdmc.ui.printer("No iLO Federation accounts found.\n")
            else:
                self.rdmc.ui.printer("iLO Federation Id list with Privileges:\n")
                if options.json:
                    outdict = dict()
                    for fed in sorted(results, key=lambda k: k["Name"]):
                        outdict[fed["Name"]] = fed["Privileges"]
                    self.rdmc.ui.print_out_json_ordered(outdict)
                else:
                    for fed in sorted(results, key=lambda k: k["Name"]):
                        privstr = ""
                        privs = fed["Privileges"]
                        for priv in privs:
                            privstr += priv + "=" + str(privs[priv]) + "\n"
                        self.rdmc.ui.printer("\nName=%s:\n%s" % (fed["Name"], privstr))

        self.cmdbase.logout_routine(self, options)
        # Return code
        return ReturnCodes.SUCCESS

    def getprivs(self, options):
        """find and return the privileges to set
        :param options: command line options
        :type options: list.
        """
        sesprivs = self.getsesprivs()
        setprivs = {}
        availableprivs = self.getsesprivs(availableprivsopts=True)

        if "UserConfigPriv" not in list(sesprivs.keys()):
            raise IdTokenError(
                "The currently logged in federation does not have the Config"
                "Privilege and cannot add or modify federations."
            )

        if options.optprivs:
            for priv in options.optprivs:
                priv = next(iter(list(priv.keys())))
                if not options.user or not options.password:
                    if priv == "SystemRecoveryConfigPriv" and self.rdmc.app.current_client.base_url == "blobstore://.":
                        raise UsernamePasswordRequiredError(
                            "Privilege %s need username and password to be specified." % priv
                        )
                if priv not in availableprivs:
                    raise IncompatibleiLOVersionError("Privilege %s is not available on this " "iLO version." % priv)

            if all(priv.values() for priv in options.optprivs):
                if (
                    any(priv for priv in options.optprivs if "SystemRecoveryConfigPriv" in priv)
                    and "SystemRecoveryConfigPriv" not in sesprivs.keys()
                ):
                    raise IdTokenError(
                        "The currently logged in account must have The System "
                        "Recovery Config privilege to add the System Recovery "
                        "Config privilege."
                    )
                else:
                    setprivs = {}
            for priv in options.optprivs:
                setprivs.update(priv)

        return setprivs

    def getsesprivs(self, availableprivsopts=False):
        """Finds and returns the current session's privileges
        :param availableprivsopts: return available privileges
        :type availableprivsopts: boolean.
        """
        if self.rdmc.app.current_client:
            sespath = self.rdmc.app.current_client.session_location
            sespath = (
                self.rdmc.app.current_client.default_prefix
                + sespath.split(self.rdmc.app.current_client.default_prefix)[-1]
            )

            ses = self.rdmc.app.get_handler(sespath, service=False, silent=True)

            if ses.status != 200:
                raise SessionExpired("Invalid session. Please logout and " "log back in or include credentials.")

            sesprivs = {
                "HostBIOSConfigPriv": True,
                "HostNICConfigPriv": True,
                "HostStorageConfigPriv": True,
                "LoginPriv": True,
                "RemoteConsolePriv": True,
                "SystemRecoveryConfigPriv": True,
                "UserConfigPriv": True,
                "VirtualMediaPriv": True,
                "VirtualPowerAndResetPriv": True,
                "iLOConfigPriv": True,
            }
            if "Oem" in ses.dict:
                sesoemhp = ses.dict["Oem"][self.rdmc.app.typepath.defs.oemhp]
                if "Privileges" in list(sesoemhp.keys()):
                    sesprivs = sesoemhp["Privileges"]
            availableprivs = list(sesprivs.keys())
            keepprivs = dict()
            for priv, val in sesprivs.items():
                if val:
                    keepprivs[priv] = sesprivs[priv]
            sesprivs = keepprivs
        else:
            sesprivs = None

        if availableprivsopts:
            return availableprivs
        else:
            return sesprivs

    def addvalidation(self, username, key, feds):
        """add validation function
        :param username: username to be added
        :type username: str.
        :param key: key to be added
        :type key: str.
        :param feds: list of federation accounts
        :type feds: list.
        """

        if len(username) >= 32:
            raise InvalidCommandLineError("User name exceeds maximum length.")
        elif len(key) >= 32 or len(key) <= 7:
            raise InvalidCommandLineError("Password is invalid length.")

    def ilofederationvalidation(self, options):
        """addfederation validation function
        :param options: command line options
        :type options: list.
        """
        self.cmdbase.login_select_validation(self, options)

    @staticmethod
    def options_addprivs_argument_group(parser):
        """Define optional arguments group
        :param parser: The parser to add the addprivs option group to
        :type parser: ArgumentParser/OptionParser
        """
        parser.add_argument(
            "--addprivs",
            dest="optprivs",
            nargs="*",
            action=_FederationParse,
            type=str,
            help="Optionally include this flag if you wish to specify "
            "which privileges you want added to the iLO federation. This overrides the default of "
            "duplicating privileges of the currently logged in federation on the new federation. "
            "Pick privileges from the privilege list in the above help text. EX: --addprivs=1,2,4",
            default=None,
        )

    @staticmethod
    def options_removeprivs_argument_group(parser):
        """Additional argument
        :param parser: The parser to add the removeprivs option group to
        :type parser: ArgumentParser/OptionParser
        """

        parser.add_argument(
            "--removeprivs",
            dest="optprivs",
            nargs="*",
            action=_FederationParse,
            type=str,
            help="Optionally include this flag if you wish to specify "
            "which privileges you want removed from the iLO federation. This overrides the default"
            " of duplicating privileges of the currently logged in federation on the new "
            "federation. Pick privileges from the privilege list in the above help text. "
            "EX: --removeprivs=1,2,4",
            default=None,
        )

    def definearguments(self, customparser):
        """Wrapper function for new command main function
        :param customparser: command line input
        :type customparser: parser.
        """
        if not customparser:
            return

        self.cmdbase.add_login_arguments_group(customparser)
        subcommand_parser = customparser.add_subparsers(dest="command")
        privilege_help = (
            "\n\nPRIVILEGES:\n\t1: Login\n\t2: Remote Console\n\t3: User Config\n\t4:"
            " iLO Config\n\t5: Virtual Media\n\t6: Virtual Power and Reset\n\n\tiLO 5 added "
            "privileges:\n\t7: Host NIC Config\n\t8: Host Bios Config\n\t9: Host Storage Config"
            "\n\t10: System Recovery Config"
        )
        # default sub-parser
        default_parser = subcommand_parser.add_parser(
            "default",
            help="Running without any sub-command will return all federation group information "
            " on the currently logged in server.",
        )
        default_parser.add_argument(
            "-j",
            "--json",
            dest="json",
            action="store_true",
            help="Optionally include this flag if you wish to change the"
            " displayed output to JSON format. Preserving the JSON data"
            " structure makes the information easier to parse.",
            default=False,
        )
        self.cmdbase.add_login_arguments_group(default_parser)  # add sub-parser
        add_help = (
            "Adds an iLO federation group to the currently logged in server. Federation "
            'group privileges may be specified with\n"--addprivs". If a federation key '
            "is not "
            "provided, the user will be prompted to provide one prior to account creation."
        )
        add_parser = subcommand_parser.add_parser(
            __subparsers__[0],
            help=add_help,
            description=add_help
            + "\n\tilofederation add [FEDERATIONNAME] [FEDERATIONKEY] "
            + privilege_help
            + "\n\tilofederation add newilofedname thisfedkey --addprivs 1,3,4",
            formatter_class=RawDescriptionHelpFormatter,
        )
        add_parser.add_argument(
            "fedname",
            help="Federation name of the federation group to add.",
            type=str,
            metavar="FEDERATION NAME",
        )
        add_parser.add_argument(
            "fedkey",
            help="Federation key of the federation group to add.",
            type=str,
            metavar="FEDERATION KEY",
        )
        self.options_addprivs_argument_group(add_parser)
        self.cmdbase.add_login_arguments_group(add_parser)

        modify_help = "Modify the privileges on an existing federation group."
        modify_parser = subcommand_parser.add_parser(
            __subparsers__[1],
            help=modify_help,
            description=modify_help + "\n\nTo add privileges:\n\tilofederation modify "
            "[FEDNAME] --addprivs <list of numbered privileges>\n\nTo remove privileges:\n\t"
            "ilofederation modify [FEDNAME] --removeprivs <list of numbered privileges>\n\n" + privilege_help,
            formatter_class=RawDescriptionHelpFormatter,
        )
        modify_parser.add_argument(
            "fedname",
            help="The federation name of the iLO account to modify.",
            metavar="FEDERATION NAME",
            type=str,
        )
        self.options_addprivs_argument_group(modify_parser)
        self.options_removeprivs_argument_group(modify_parser)
        self.cmdbase.add_login_arguments_group(modify_parser)

        # changepass sub-parser
        changekey_help = "Change the key of an iLO federation group on the currently logged in " "server."
        changekey_parser = subcommand_parser.add_parser(
            __subparsers__[2],
            help=changekey_help,
            description=changekey_help + "\n\nexample:ilofederation changekey [FEDNAME] [NEWFEDKEY]",
            formatter_class=RawDescriptionHelpFormatter,
        )
        changekey_parser.add_argument(
            "fedname",
            help="The iLO federation account to be updated with a new federation key (password).",
            metavar="FEDERATION NAME",
            type=str,
        )
        changekey_parser.add_argument(
            "fedkey",
            help="The federation key (password) to be altered for the selected iLO federation "
            " account. If you do not include a federation key, you will be prompted to enter one.",
            metavar="FEDERATION KEY",
            type=str,
            nargs="?",
            default="",
        )
        self.cmdbase.add_login_arguments_group(changekey_parser)

        # delete sub-parser
        delete_help = "Deletes the provided iLO user account on the currently logged in server."
        delete_parser = subcommand_parser.add_parser(
            __subparsers__[3],
            help=delete_help,
            description=delete_help + "\n\nexample: ilofederation delete fedname",
            formatter_class=RawDescriptionHelpFormatter,
        )
        delete_parser.add_argument(
            "fedname",
            help="The iLO federation account to delete.",
            metavar="FEDERATION NAME",
            type=str,
        )
        self.cmdbase.add_login_arguments_group(delete_parser)



###
# Copyright 2016-2021 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

# -*- coding: utf-8 -*-
""" Status Command for RDMC """

from functools import reduce

from redfish.ris.utils import merge_dict

try:
    from rdmc_helper import InvalidCommandLineErrorOPTS, ReturnCodes
except ImportError:
    from ilorest.rdmc_helper import InvalidCommandLineErrorOPTS, ReturnCodes


class StatusCommand:
    """Constructor"""

    def __init__(self):
        self.ident = {
            "name": "status",
            "usage": None,
            "description": "Run to display all pending changes within"
            " the currently\n\tselected type that need to be"
            " committed\n\texample: status",
            "summary": "Displays all pending changes within a selected type" " that need to be committed.",
            "aliases": [],
            "auxcommands": ["SelectCommand"],
        }
        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()

    def run(self, line, help_disp=False):
        """Main status worker function

        :param line: command line input
        :type line: string.
        """
        if help_disp:
            self.parser.print_help()
            return ReturnCodes.SUCCESS
        try:
            (options, _) = self.rdmc.rdmc_parse_arglist(self, line)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineErrorOPTS("")

        self.statusvalidation(options)
        contents = self.rdmc.app.status()
        selector = self.rdmc.app.selector

        if contents and options.json:
            self.jsonout(contents)
        elif contents:
            self.outputpatches(contents, selector)
        else:
            self.rdmc.ui.printer("No changes found\n")

        # Return code
        return ReturnCodes.SUCCESS

    def jsonout(self, contents):
        """Helper function to print json output of patches

        :param contents: contents for the selection
        :type contents: string.
        """
        self.rdmc.ui.printer("Current changes found:\n")
        createdict = lambda y, x: {x: y}
        totdict = {}
        for item in contents:
            for keypath, value in item.items():
                path = keypath.split("(")[1].strip("()")
                cont = {}
                totdict[path] = cont
                for content in value:
                    val = (
                        ["List Manipulation"]
                        if content["op"] == "move"
                        else [content["value"].strip("\"'")] if len(content["value"]) else [""]
                    )
                    cont = reduce(
                        createdict,
                        reversed([path] + content["path"].strip("/").split("/") + val),
                    )
                    merge_dict(totdict, cont)
        self.rdmc.ui.print_out_json(totdict)

    def outputpatches(self, contents, selector):
        """Helper function for status for use in patches

        :param contents: contents for the selection
        :type contents: string.
        :param selector: type selected
        :type selector: string.
        """
        self.rdmc.ui.printer("Current changes found:\n")
        for item in contents:
            moveoperation = ""
            for key, value in item.items():
                if selector and key.lower().startswith(selector.lower()):
                    self.rdmc.ui.printer("%s (Currently selected)\n" % key)
                else:
                    self.rdmc.ui.printer("%s\n" % key)

                for content in value:
                    try:
                        if content["op"] == "move":
                            moveoperation = "/".join(content["path"].split("/")[1:-1])
                            continue
                    except:
                        if content[0]["op"] == "move":
                            moveoperation = "/".join(content[0]["path"].split("/")[1:-1])
                            continue
                    try:
                        if isinstance(content[0]["value"], int):
                            self.rdmc.ui.printer("\t%s=%s" % (content[0]["path"][1:], content[0]["value"]))
                        elif not isinstance(content[0]["value"], bool) and not len(content[0]["value"]) == 0:
                            if content[0]["value"][0] == '"' and content[0]["value"][-1] == '"':
                                self.rdmc.ui.printer(
                                    "\t%s=%s"
                                    % (
                                        content[0]["path"][1:],
                                        content[0]["value"][1:-1],
                                    )
                                )
                            else:
                                self.rdmc.ui.printer("\t%s=%s" % (content[0]["path"][1:], content[0]["value"]))
                        else:
                            output = content[0]["value"]

                            if not isinstance(output, bool):
                                if len(output) == 0:
                                    output = '""'

                            self.rdmc.ui.printer("\t%s=%s" % (content[0]["path"][1:], output))
                    except:
                        if isinstance(content["value"], int):
                            self.rdmc.ui.printer("\t%s=%s" % (content["path"][1:], content["value"]))
                        elif (
                            content["value"]
                            and not isinstance(content["value"], bool)
                            and not len(content["value"]) == 0
                        ):
                            if content["value"][0] == '"' and content["value"][-1] == '"':
                                self.rdmc.ui.printer("\t%s=%s" % (content["path"][1:], content["value"]))
                            else:
                                self.rdmc.ui.printer("\t%s=%s" % (content["path"][1:], content["value"]))
                        else:
                            output = content["value"]

                            if output and not isinstance(output, bool):
                                if len(output) == 0:
                                    output = '""'

                            self.rdmc.ui.printer("\t%s=%s" % (content["path"][1:], output))
                    self.rdmc.ui.printer("\n")
            if moveoperation:
                self.rdmc.ui.printer("\t%s=List Manipulation\n" % moveoperation)

    def statusvalidation(self, options):
        """Status method validation function"""

        self.cmdbase.login_select_validation(self, options)

    def definearguments(self, customparser):
        """Wrapper function for new command main function

        :param customparser: command line input
        :type customparser: parser.
        """
        if not customparser:
            return

        self.cmdbase.add_login_arguments_group(customparser)

        customparser.add_argument(
            "-j",
            "--json",
            dest="json",
            action="store_true",
            help="Optionally include this flag if you wish to change the"
            " displayed output to JSON format. Preserving the JSON data"
            " structure makes the information easier to parse.",
            default=False,
        )



###
# Copyright 2016 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###
# -*- coding: utf-8 -*-
""" Security Status Command for rdmc """

import struct
from argparse import SUPPRESS
from ctypes import POINTER, c_char_p, c_ubyte, create_string_buffer

try:
    from rdmc_base_classes import RdmcCommandBase
    from rdmc_helper import Encryption, InvalidCommandLineErrorOPTS, ReturnCodes
except ImportError:
    from ilorest.rdmc_base_classes import RdmcCommandBase
    from ilorest.rdmc_helper import Encryption, InvalidCommandLineErrorOPTS, ReturnCodes
from redfish.hpilo.risblobstore2 import BlobStore2
from redfish.hpilo.rishpilo import BlobReturnCodes, HpIloChifAccessDeniedError


class SecurityStatusCommand(RdmcCommandBase):
    """Security Status class command"""

    def __init__(self):
        self.ident = {
            "name": "securitystatus",
            "usage": None,
            "description": "Security status example:\n\tsecuritystatus",
            "summary": "command to retrieve the current system security status and validate " "credentials via chif.",
            "aliases": [],
            "auxcommands": [],
        }
        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()

    def run(self, line, help_disp=False):
        """Access blobstore directly and perform desired function

        :param line: string of arguments passed in
        :type line: str.
        """
        if help_disp:
            self.parser.print_help()
            return ReturnCodes.SUCCESS
        try:
            (options, _) = self.rdmc.rdmc_parse_arglist(self, line)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineErrorOPTS("")

        if bool(options.user) ^ bool(options.password):
            self.rdmc.ui.error("Credentials: Missing\n")

        elif options.user and options.password:
            if options.encode:
                options.user = Encryption.decode_credentials(options.user)
                if isinstance(options.user, bytes):
                    options.user = options.user.decode("utf-8")
                options.password = Encryption.decode_credentials(options.password)
                if isinstance(options.password, bytes):
                    options.password = options.password.decode("utf-8")

            result = self.validate_creds(options.user, options.password)
            # self.rdmc.ui.printer("Validate Creds is {}...\n".format(result))
            if result:
                secstate = BlobStore2().get_security_state()
                if isinstance(secstate, bytes):
                    if secstate == b"\x00":
                        state = struct.unpack("B", secstate)
                        state = str(state).strip(",( )")
                        self.rdmc.ui.printer("Security State is {}\n".format(state))
                    else:
                        secstate = secstate.decode("utf-8")
                        self.rdmc.ui.printer("Security State is {}\n".format(secstate))
                else:
                    self.rdmc.ui.printer("Security State is {}\n".format(secstate))
                self.rdmc.ui.printer("Credentials: Valid\n")
            else:
                self.rdmc.ui.error("Credentials: Invalid\n")

        else:
            secstate = BlobStore2().get_security_state()
            if isinstance(secstate, bytes):
                if secstate == b"\x00":
                    state = struct.unpack("B", secstate)
                    state = str(state).strip(",( )")
                    self.rdmc.ui.printer("Security State is {}\n".format(state))
                else:
                    secstate = secstate.decode("utf-8")
                    self.rdmc.ui.printer("Security State is {}\n".format(secstate))
            else:
                self.rdmc.ui.printer("Security State is {}\n".format(secstate))

        return ReturnCodes.SUCCESS

    def validate_creds(self, user, passwrd):
        """Validates credentials via CHIF

        :param user: username to validate
        :type user: str.
        :param passwrd: password to validate
        :type passwrd: str.
        """

        valid = False
        dll = BlobStore2.gethprestchifhandle()
        dll.ChifInitialize(None)
        sec_support = dll.ChifGetSecuritySupport()
        if sec_support <= 1:
            dll.ChifEnableSecurity()
        dll.initiate_credentials.argtypes = [c_char_p, c_char_p]
        dll.initiate_credentials.restype = POINTER(c_ubyte)

        usernew = create_string_buffer(user.encode("utf-8"))
        passnew = create_string_buffer(passwrd.encode("utf-8"))

        dll.initiate_credentials(usernew, passnew)
        credreturn = dll.ChifVerifyCredentials()
        if credreturn == 0:
            valid = True
        else:
            valid = False
            if not credreturn == BlobReturnCodes.CHIFERR_AccessDenied:
                raise HpIloChifAccessDeniedError(
                    "Error %s - Chif Access Denied occurred while trying " "to open a channel to iLO." % credreturn
                )

        BlobStore2.unloadchifhandle(dll)
        return valid

    def definearguments(self, customparser):
        """Wrapper function for new command main function

        :param customparser: command line input
        :type customparser: parser.
        """
        if not customparser:
            return

        customparser.add_argument(
            "-u",
            "--user",
            dest="user",
            help="If you are not logged in yet, including this flag along"
            " with the password and URL flags can be used to login to a"
            " server in the same command."
            "",
            default=None,
        )
        customparser.add_argument(
            "-p",
            "--password",
            dest="password",
            help="""Use the provided iLO password to log in.""",
            default=None,
        )
        customparser.add_argument(
            "-e",
            "--enc",
            dest="encode",
            action="store_true",
            help=SUPPRESS,
            default=False,
        )



###
# Copyright 2016-2021 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

# -*- coding: utf-8 -*-
""" Certificates Command for rdmc """

import time
from argparse import RawDescriptionHelpFormatter

try:
    from rdmc_helper import (
        IloLicenseError,
        IncompatibleiLOVersionError,
        InvalidCommandLineErrorOPTS,
        InvalidFileInputError,
        NoContentsFoundForOperationError,
        ReturnCodes,
        ScepenabledError,
    )
except ImportError:
    from ilorest.rdmc_helper import (
        IloLicenseError,
        IncompatibleiLOVersionError,
        InvalidCommandLineErrorOPTS,
        InvalidFileInputError,
        NoContentsFoundForOperationError,
        ReturnCodes,
        ScepenabledError,
    )

__filename__ = "certificate.txt"

from redfish.ris import IdTokenError


class CertificateCommand:
    """Commands Certificates actions to the server"""

    def __init__(self):
        self.ident = {
            "name": "certificate",
            "usage": None,
            "description": "Generate a certificate signing request (CSR) or import an X509 formatted"
            " TLS or CA certificate.\nImport Scep Certificate.\nInvoke Auto Enroll of certificate generation\n\n"
            "NOTE: Use quotes to include parameters which contain whitespace when "
            'generating a CSR.\nexample: certificate gen_csr "Hewlett Packard Enterprise"'
            '"iLORest Group" "CName"\n"United States" "Texas" "Houston" "False or True"\n\n'
            "NOTE: unifiedcertificate command which was augmenting certificate command is merged"
            " into single command certificate",
            "summary": "Command for importing both iLO and login authorization "
            "certificates as well as generating iLO certificate signing requests (CSR)\n",
            "aliases": ["unifiedcertificate"],
            "auxcommands": [],
        }
        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()
        self.view = None
        self.importcert = None
        self.exportcert = None
        self.delete = None
        self.gencsr = None
        self.autoenroll = None

    def run(self, line, help_disp=False):
        """Main Certificates Command function

        :param options: list of options
        :type options: list.
        :param help_disp: display help flag
        :type line: bool.
        """
        if help_disp:
            line.append("-h")
            try:
                (_, _) = self.rdmc.rdmc_parse_arglist(self, line)
            except:
                return ReturnCodes.SUCCESS
            return ReturnCodes.SUCCESS
        try:
            (options, _) = self.rdmc.rdmc_parse_arglist(self, line)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineErrorOPTS("")

        self.certificatesvalidation(options)

        returnCode = None

        if options.command == "csr":
            self.rdmc.ui.printer(
                "This command has been deprecated. Please use the following command: "
                'certificate gen_csr "Hewlet Packard Enterprice" "ILORestGroup" ...\n'
                "It performs the same function.\n"
                "Check --help for more information.\n"
            )
            self.cmdbase.logout_routine(self, options)
            # Return code
            return returnCode
        elif options.command == "ca":
            self.rdmc.ui.printer(
                "This command has been deprecated. Please use the following "
                "command: certificate import --ca_cert <ca_txt>\n"
                "It performs the same function.\n"
                "Check --help for more information.\n"
            )
            self.cmdbase.logout_routine(self, options)
            # Return code
            return returnCode
        elif options.command == "getcsr":
            returnCode = self.get_csr_helper(options)
        elif options.command == "crl":
            self.rdmc.ui.printer(
                "This command has been deprecated. Please use the following command: "
                "certificate import --crl_cert <crl_txt>\n"
                "It performs the same function.\n"
                "Check --help for more information.\n"
            )
            self.cmdbase.logout_routine(self, options)
            # Return code
            return returnCode
        elif options.command == "tls":
            self.rdmc.ui.printer(
                "This command has been deprecated. Please use the following command: "
                "certificate import --tls_cert <tls_txt>\n"
                "It performs the same function.\n"
                "Check --help for more information.\n"
            )
            self.cmdbase.logout_routine(self, options)
            # Return code
            return returnCode
        if "view" in options.command.lower():
            self.view = True
            self.importcert = False
            self.delete = False
            self.gencsr = False
            self.autoenroll = False
            self.exportcert = False
        elif "import" in options.command.lower():
            self.view = False
            self.importcert = True
            self.delete = False
            self.gencsr = False
            self.autoenroll = False
            self.exportcert = False
        elif "export" in options.command.lower():
            self.view = False
            self.importcert = False
            self.delete = False
            self.gencsr = False
            self.autoenroll = False
            self.exportcert = True
        elif "delete" in options.command.lower():
            self.view = False
            self.importcert = False
            self.delete = True
            self.gencsr = False
            self.autoenroll = False
            self.exportcert = False
        elif "gen_csr" in options.command.lower():
            self.view = False
            self.importcert = False
            self.delete = False
            self.gencsr = True
            self.autoenroll = False
            self.exportcert = False
        elif "auto_enroll" in options.command.lower():
            self.view = False
            self.importcert = False
            self.delete = False
            self.gencsr = False
            self.autoenroll = True
            self.exportcert = False

        if self.view:
            returnCode = self.viewfunction(options)
        elif self.importcert:
            returnCode = self.importfunction(options)
        elif self.exportcert:
            returnCode = self.exportfunction(options)
        elif self.delete:
            returnCode = self.deletefunction(options)
        elif self.gencsr:
            returnCode = self.gencsrfunction(options)
        elif self.autoenroll:
            if self.rdmc.app.typepath.defs.isgen10:
                returnCode = self.autoenrollfunction(options)
            else:
                self.rdmc.ui.printer("Gen 9 doesnt support this feature\n")
                returnCode = ReturnCodes.SUCCESS

        self.cmdbase.logout_routine(self, options)
        # Return code
        return returnCode

    def autoenrollfunction(self, options):
        """Automatic Scep cert enrollement process

        :param options: list of options
        :type options: list.
        """
        select = self.rdmc.app.typepath.defs.securityservice
        results = self.rdmc.app.select(selector=select)

        try:
            results = results[0]
        except:
            pass

        if results:
            path = results.resp.request.path
        else:
            raise NoContentsFoundForOperationError("Unable to find %s" % select)

        bodydict = results.resp.dict

        try:
            for item in bodydict["Links"]:
                if "AutomaticCertificateEnrollment" in item:
                    path = bodydict["Links"]["AutomaticCertificateEnrollment"]["@odata.id"]
                    break
        except:
            path = path + "AutomaticCertificateEnrollment"

        body = {
            "AutomaticCertificateEnrollmentSettings": {
                "ServiceEnabled": eval(options.autoenroll_ScepService.strip('"')),
                "ServerUrl": options.autoenroll_scep_enrollAddress.strip('"'),
                "ChallengePassword": options.autoenroll_challengepassword.strip('"'),
            },
            "HttpsCertCSRSubjectValue": {
                "OrgName": options.autoenroll_orgname.strip('"'),
                "OrgUnit": options.autoenroll_orgunit.strip('"'),
                "CommonName": options.autoenroll_commonname.strip('"'),
                "Country": options.autoenroll_country.strip('"'),
                "State": options.autoenroll_state.strip('"'),
                "City": options.autoenroll_city.strip('"'),
                "IncludeIP": eval(options.autoenroll_includeIP.strip('"')),
            },
        }

        try:
            results = self.rdmc.app.patch_handler(path, body, silent=True)

            i = 0
            results1 = self.rdmc.app.get_handler(path, silent=True)
            while i < 9 and (
                results1.dict["AutomaticCertificateEnrollmentSettings"]["CertificateEnrollmentStatus"] == "InProgress"
            ):
                results1 = self.rdmc.app.get_handler(path, silent=True)
                time.sleep(1)
                i = i + 1

            if results.status == 200 and (
                not (results1.dict["AutomaticCertificateEnrollmentSettings"]["CertificateEnrollmentStatus"] == "Failed")
            ):
                self.rdmc.ui.printer("Auto enrolment Enable/Disable operation successful \n")
                return ReturnCodes.SUCCESS
            elif results.status == 400:
                self.rdmc.ui.error(
                    "There was a problem with auto enroll, Please check whether Scep CA cert is imported \t\n"
                )

                return ReturnCodes.SCEP_ENABLED_ERROR
            else:
                self.rdmc.ui.error(
                    "There was a problem with auto enroll, Plese Check the Url/Password whether it is correct\n"
                )
                return ReturnCodes.SCEP_ENABLED_ERROR
        except IloLicenseError:
            self.rdmc.ui.error("License Error Occured while auto enroll\n")
            return ReturnCodes.ILO_LICENSE_ERROR
        except IdTokenError:
            self.rdmc.ui.printer("Insufficient Privilege to auto enroll scep certificate process\n")
            return ReturnCodes.RIS_MISSING_ID_TOKEN

    def gencsrfunction(self, options):
        """Main Certificates Command function

        :param options: list of options
        :type options: list.
        """
        try:
            select = self.rdmc.app.typepath.defs.hphttpscerttype
            results = self.rdmc.app.select(selector=select)
            try:
                results = results[0]
            except:
                pass

            if results:
                path = results.resp.request.path
            else:
                raise NoContentsFoundForOperationError("Unable to find %s" % select)

            bodydict = results.resp.dict

            try:
                for item in bodydict["Actions"]:
                    if "GenerateCSR" in item:
                        if self.rdmc.app.typepath.defs.isgen10:
                            action = item.split("#")[-1]
                        else:
                            action = "GenerateCSR"

                        path = bodydict["Actions"][item]["target"]
                        break
            except:
                action = "GenerateCSR"
        except:
            path = "redfish/v1/Managers/1/SecurityService/HttpsCert"
            action = "GenerateCSR"

        body = {
            "Action": action,
            "OrgName": options.gencsr_orgname.strip('"'),
            "OrgUnit": options.gencsr_orgunit.strip('"'),
            "CommonName": options.gencsr_commonname.strip('"'),
            "Country": options.gencsr_country.strip('"'),
            "State": options.gencsr_state.strip('"'),
            "City": options.gencsr_city.strip('"'),
            "IncludeIP": eval(options.gencsr_parser_includeIP.strip('"')),
        }

        try:
            results = self.rdmc.app.post_handler(path, body)
            if results.status == 200:
                return ReturnCodes.SUCCESS
        except ScepenabledError:
            self.rdmc.ui.printer("SCEP is enabled , CSR cant be generated \n")
            return ReturnCodes.SCEP_ENABLED_ERROR
        except IloLicenseError:
            self.rdmc.ui.error("License Error Occured while generating CSR")
            return ReturnCodes.ILO_LICENSE_ERROR
        except IdTokenError:
            self.rdmc.ui.printer("Insufficient Privilege to generate CSR\n")
            return ReturnCodes.RIS_MISSING_ID_TOKEN

    def deletefunction(self, options):
        """Certificate Delete Function

        :param options: list of options
        :type options: list.
        """

        select = self.rdmc.app.typepath.defs.hphttpscerttype
        results = self.rdmc.app.select(selector=select)

        try:
            results = results[0]
        except:
            pass

        if results:
            path = results.resp.request.path
        else:
            raise NoContentsFoundForOperationError("Unable to find %s" % select)

        try:
            results = self.rdmc.app.delete_handler(path, silent=True)
            if results.status == 200:
                self.rdmc.ui.printer("Deleted the https certifcate successfully\n")
                return ReturnCodes.SUCCESS
            if results.status == 403:
                self.rdmc.ui.error("Insufficient Privilege to delete certificate\n")
                return ReturnCodes.RIS_MISSING_ID_TOKEN
            if results.status == 400:
                self.rdmc.ui.error("SCEP is enabled , Cant delete the certificate\n")
                return ReturnCodes.SCEP_ENABLED_ERROR
        except IloLicenseError:
            self.rdmc.ui.error("License Error Occured while delete")
            return ReturnCodes.ILO_LICENSE_ERROR
        except IncompatibleiLOVersionError:
            self.rdmc.ui.error("iLO FW version on this server doesnt support this operation")
            return ReturnCodes.INCOMPATIBLE_ILO_VERSION_ERROR

    def viewfunction(self, options):
        """View scep certifcates or https certificates

        :param options: list of options
        :type options: list.
        """

        if options.scep_cert:
            if self.rdmc.app.typepath.defs.isgen10:
                self.view_scepcertificate()  # View scep certificates
            else:
                self.rdmc.ui.printer("Feature not supported on Gen 9\n")
                return ReturnCodes.SUCCESS
        if options.https_cert:
            self.view_httpscertificate()  # View https certificates

    def view_scepcertificate(self):
        """
        View Scep certificate
        """
        select = self.rdmc.app.typepath.defs.securityservice
        results = self.rdmc.app.select(selector=select)

        try:
            results = results[0]
        except:
            pass

        if results:
            path = results.resp.request.path
        else:
            raise NoContentsFoundForOperationError("Unable to find %s" % select)

        bodydict = results.resp.dict

        try:
            for item in bodydict["Links"]:
                if "AutomaticCertificateEnrollment" in item:
                    path = bodydict["Links"]["AutomaticCertificateEnrollment"]["@odata.id"]
                    break
        except:
            path = path + "AutomaticCertificateEnrollment"

        try:
            results = self.rdmc.app.get_handler(path, silent=True)
            if results.status == 200:
                self.rdmc.ui.printer("Scep Certificate details ...\n")
                results = results.dict
                self.print_cert_info(results)
                return ReturnCodes.SUCCESS

        except IloLicenseError:
            self.rdmc.ui.error("Error Occured while Uninstall")
            return ReturnCodes.ILO_LICENSE_ERROR

    def print_cert_info(self, results):
        """
        Prints the cert info
        """
        for key, value in results.items():
            if "@odata" not in key:
                if type(value) is dict:
                    self.print_cert_info(value)
                else:
                    self.rdmc.ui.printer(key + ":" + str(value) + "\n")

    def view_httpscertificate(self):
        """
        View Https certificate
        """
        try:
            select = self.rdmc.app.typepath.defs.hphttpscerttype
            results = self.rdmc.app.select(selector=select)

            try:
                results = results[0]
            except:
                pass

            if results:
                path = results.resp.request.path
            else:
                raise NoContentsFoundForOperationError("Unable to find %s" % select)
        except:
            path = "redfish/v1/Managers/1/SecurityService/HttpsCert"

        try:
            results = self.rdmc.app.get_handler(path, silent=True)
            if results.status == 200:
                self.rdmc.ui.printer("Https Certificate details ...\n")
                results = results.dict
                self.print_cert_info(results)
                return ReturnCodes.SUCESS
        except IloLicenseError:
            self.rdmc.ui.error("Error Occured while Uninstall")
            return ReturnCodes.ILO_LICENSE_ERROR

    def exportfunction(self, options):
        result = self.exportplatformhelper(options)

        if result:
            if options.filename:
                self.file_handler(next(iter(options.filename)), result, options, "wb")
                self.rdmc.ui.printer("The certificate was saved to: %s\n" % next(iter(options.filename)))
                return ReturnCodes.SUCCESS
            else:
                self.rdmc.ui.printer("The certificate retrieved is as follows:\n%s\n" % result)
                return ReturnCodes.SUCCESS
        else:
            raise NoContentsFoundForOperationError("An error occurred retrieving the requested certificate.\n")

    def exportplatformhelper(self, options):
        """Helper function for exporting a platform certificate
        :param options: list of options
        :type options: list.
        """

        str_type = "PlatformCert"
        ss_instance = next(iter(self.rdmc.app.select("SecurityService." + ".", path_refresh=True)))
        if options.ldevid_cert:
            str_type = "iLOLDevID"
        elif options.idevid_cert:
            str_type = "iLOIDevID"
        elif options.systemiak_cert:
            str_type = "SystemIAK"
        elif options.systemidevid_cert:
            str_type = "SystemIDevID"
        instance_path_uri = (
            (ss_instance.dict[str_type]["Certificates"][self.rdmc.app.typepath.defs.hrefstring])
            if ss_instance.dict.get("SystemIAK")
            else None
        )
        instance_data = self.rdmc.app.get_handler(instance_path_uri, silent=True)
        cert = None
        if instance_data.dict.get("Members"):
            cert = self.rdmc.app.get_handler(
                instance_data.dict["Members"][getattr(options, "id", 0) - 1].get(
                    self.rdmc.app.typepath.defs.hrefstring
                ),
                silent=True,
            ).dict
            return cert.get("CertificateString")
        else:
            raise NoContentsFoundForOperationError(
                "Unable to find specified certificate at " "position %s." % getattr(options, "id", 0)
            )

    def file_handler(self, filename, data, options, operation="rb"):
        """
        Wrapper function to read or write data to a respective file
        :param data: data to be written to output file
        :type data: container (list of dictionaries, dictionary, etc.)
        :param file: filename to be written
        :type file: string (generally this should be self.clone_file or tmp_clone_file
        :param operation: file operation to be performed
        :type operation: string ('w+', 'a+', 'r+')
        :param options: command line options
        :type options: attribute
        :returns: json file data
        """
        writeable_ops = ["wb", "w", "w+", "a", "a+"]

        if operation in writeable_ops:
            with open(filename, operation) as fh:
                try:
                    data = data.encode("UTF-8")
                except IOError:
                    raise InvalidFileInputError("Unable to write to file '%s'" % filename)
                except UnicodeEncodeError:
                    pass
                finally:
                    fh.write(data)
        else:
            with open(filename, operation) as fh:
                fdata = fh.read()
                try:
                    return fdata.decode("UTF-8")
                except UnicodeDecodeError:
                    return fdata
                except IOError:
                    raise InvalidFileInputError("Unable to read from file '%s'" % filename)

    def importfunction(self, options):
        if self.rdmc.app.typepath.defs.isgen10:
            return self.importfunctionhelper(options)
        else:
            self.rdmc.ui.printer("Gen 9 doesnt support this feature\n")
            return ReturnCodes.SUCCESS

    def importscephelper(self, options):
        """
        Import Scep certificate
        """
        select = self.rdmc.app.typepath.defs.securityservice
        results = self.rdmc.app.select(selector=select)

        try:
            results = results[0]
        except:
            pass

        if results:
            path = results.resp.request.path
        else:
            raise NoContentsFoundForOperationError("Unable to find %s" % select)

        bodydict = results.resp.dict

        try:
            for item in bodydict["Links"]:
                if "AutomaticCertificateEnrollment" in item:
                    path = bodydict["Links"]["AutomaticCertificateEnrollment"]["@odata.id"]
                    break
        except:
            path = path + "AutomaticCertificateEnrollment"

        path = path + "Actions/HpeAutomaticCertEnrollment.ImportCACertificate"

        action = "HpeAutomaticCertEnrollment.ImportCACertificate"

        certdata = None
        scep_CACert = options.certfile

        try:
            with open(scep_CACert) as certfile:
                certdata = certfile.read()
                certfile.close()
        except:
            pass

        body = {"Action": action, "Certificate": certdata}

        try:
            result = self.rdmc.app.post_handler(path, body)
            if result.status == 200:
                self.rdmc.ui.printer("Imported the scep certificate successfully\n")
                return ReturnCodes.SUCCESS
        except IdTokenError:
            self.rdmc.ui.printer("Insufficient Privilege to import scep CA certificate\n")
            return ReturnCodes.RIS_MISSING_ID_TOKEN
        except IloLicenseError:
            self.rdmc.ui.error("Error Occured while importing scep certificate")
            return ReturnCodes.ILO_LICENSE_ERROR
        except IncompatibleiLOVersionError:
            self.rdmc.ui.error("iLO FW version on this server doesnt support this operation")
            return ReturnCodes.INCOMPATIBLE_ILO_VERSION_ERROR

    def importfunctionhelper(self, options):
        result = None
        if getattr(options, "scep_cert"):
            result = self.importscephelper(options)
        elif getattr(options, "ca_cert"):
            result = self.importcahelper(options)
        elif getattr(options, "crl_cert"):
            result = self.importcrlhelper(options)
        elif getattr(options, "tls_cert"):
            result = self.importtlshelper(options)
        elif (
            getattr(options, "ldevid_cert")
            or getattr(options, "idevid_cert")
            or getattr(options, "systemiak_cert")
            or getattr(options, "systemidevid_cert")
            or getattr(options, "platform_cert")
        ):
            result = self.importplatformhelper(options)

        return result

    def generatecerthelper(self, options):
        """Main Certificates Command function

        :param options: list of options
        :type options: list.
        """
        self.rdmc.ui.printer(
            "Warning:Command CSR has been replaced with Gen_csr comamnd and "
            "CSR command will be deprecated in next release of our tool. Please plan to use gen_csr command instead \n"
        )
        select = self.rdmc.app.typepath.defs.hphttpscerttype
        results = self.rdmc.app.select(selector=select)

        try:
            results = results[0]
        except:
            pass

        if results:
            path = results.resp.request.path
        else:
            raise NoContentsFoundForOperationError("Unable to find %s" % select)

        bodydict = results.resp.dict

        try:
            for item in bodydict["Actions"]:
                if "GenerateCSR" in item:
                    if self.rdmc.app.typepath.defs.isgen10:
                        action = item.split("#")[-1]
                    else:
                        action = "GenerateCSR"

                    path = bodydict["Actions"][item]["target"]
                    break
        except:
            action = "GenerateCSR"

        body = {
            "Action": action,
            "OrgName": options.csr_orgname.strip('"'),
            "OrgUnit": options.csr_orgunit.strip('"'),
            "CommonName": options.csr_commonname.strip('"'),
            "Country": options.csr_country.strip('"'),
            "State": options.csr_state.strip('"'),
            "City": options.csr_city.strip('"'),
        }

        self.rdmc.ui.printer(
            "iLO is creating a new certificate signing request. " "This process can take up to 10 minutes.\n"
        )

        try:
            self.rdmc.app.post_handler(path, body)
            return ReturnCodes.SUCCESS
        except ScepenabledError:
            self.rdmc.ui.printer("SCEP is enabled , operation not allowed \n")
            return ReturnCodes.SCEP_ENABLED_ERROR

    def getcerthelper(self, options):
        """Helper function for importing CRL certificate

        :param options: list of options
        :type options: list.
        """

        select = self.rdmc.app.typepath.defs.hphttpscerttype
        results = self.rdmc.app.select(selector=select, path_refresh=True)

        try:
            results = results[0]
        except:
            pass

        if results:
            try:
                csr = results.resp.dict["CertificateSigningRequest"]
                if not csr:
                    raise ValueError
            except (KeyError, ValueError):
                raise NoContentsFoundForOperationError(
                    "Unable to find a valid certificate. If "
                    "you just generated a new certificate "
                    "signing request the process may take "
                    "up to 10 minutes."
                )

            if not options.filename:
                filename = __filename__
            else:
                filename = options.filename[0]

            outfile = open(filename, "w")
            outfile.write(csr)
            outfile.close()

            self.rdmc.ui.printer("Certificate saved to: %s\n" % filename)
            return ReturnCodes.SUCCESS
        else:
            raise NoContentsFoundForOperationError("Unable to find %s" % select)

    def get_csr_helper(self, options):
        result = None
        if options.TLSCERT:
            instance = next(iter(self.rdmc.app.select("HttpsCert.", path_refresh=True)))
            result = instance.dict.get("CertificateSigningRequest")
        elif options.PLATFORM:
            tmp = self.gen_csr_helper(options)
            if tmp:
                result = tmp.dict.get("CSRString")

        if result:
            if not options.filename:
                filename = __filename__
            else:
                filename = options.filename[0]

            outfile = open(filename, "w")
            outfile.write(result)
            outfile.close()
            self.rdmc.ui.printer("Certificate saved to: %s\n" % filename)
            return ReturnCodes.SUCCESS
        else:
            self.rdmc.ui.error(
                "An error occurred retrieving a CSR. Check whether CSR is requested , "
                "if not kindly run gen_csr command and generate CSR"
            )

    def gen_csr_helper(self, options):
        """
        :param options: list of options
        :type options: attributes.
        """
        body = None
        path = None
        action = "GenerateCSR"
        if options.TLSCERT:
            instance = next(iter(self.rdmc.app.select("HttpsCert.", path_refresh=True)))
            body = {
                "Action": action,
                "OrgName": options.csr_orgname.strip('"'),
                "OrgUnit": options.csr_orgunit.strip('"'),
                "CommonName": options.csr_commonname.strip('"'),
                "Country": options.csr_country.strip('"'),
                "State": options.csr_state.strip('"'),
                "City": options.csr_city.strip('"'),
            }

        elif options.PLATFORM:
            # There is seemingly no way to update the Certificate subject data which
            # appears problematic.
            _ = next(iter(self.rdmc.app.select("CertificateService.", path_refresh=True)))
            ss_instance = next(iter(self.rdmc.app.select("SecurityService.", path_refresh=True)))
            cert_obtain_path = ss_instance.dict.get("iLOLDevID")[next(iter(ss_instance.dict.get("iLOLDevID")))].get(
                self.rdmc.app.typepath.defs.hrefstring
            )
            if not cert_obtain_path:
                raise NoContentsFoundForOperationError("Unable to find specified certificate path" " for CSR request")
            instance = next(iter(self.rdmc.app.select("CertificateService.", path_refresh=True)))
            body = {"CertificateCollection": {self.rdmc.app.typepath.defs.hrefstring: cert_obtain_path}}

        try:
            for act in instance.dict.get("Actions"):
                if "GenerateCSR" in act:
                    if self.rdmc.app.typepath.defs.isgen10:
                        action = act.split("#")[-1]
                    else:
                        action = "GenerateCSR"
                    path = instance.dict["Actions"][act]["target"]
                    break
        except:
            raise NoContentsFoundForOperationError("Unable to find specified certificate action" "path for CSR request")

        self.rdmc.ui.printer(
            "iLO is creating a new certificate signing request. " "This request may take up to 10 minutes.\n"
        )
        return self.rdmc.app.post_handler(path, body)

    def importtlshelper(self, options):
        """Helper function for importing TLS certificate

        :param options: list of options
        :type options: list.
        """
        tlsfile = options.certfile

        try:
            with open(tlsfile) as certfile:
                certdata = certfile.read()
                certfile.close()
        except:
            raise InvalidFileInputError("Error loading the specified file.")

        select = self.rdmc.app.typepath.defs.hphttpscerttype
        results = self.rdmc.app.select(selector=select)

        try:
            results = results[0]
        except:
            pass

        if results:
            path = results.resp.request.path
        else:
            raise NoContentsFoundForOperationError("Unable to find %s" % select)

        bodydict = results.resp.dict
        try:
            for item in bodydict["Actions"]:
                if "ImportCertificate" in item:
                    if self.rdmc.app.typepath.defs.isgen10:
                        action = item.split("#")[-1]
                    else:
                        action = "ImportCertificate"
                    path = bodydict["Actions"][item]["target"]
                    break
        except:
            action = "ImportCertificate"

        body = {"Action": action, "Certificate": certdata}

        try:
            result = self.rdmc.app.post_handler(path, body)
            if result.status == 200:
                self.rdmc.ui.printer("Imported the TLS certificate successfully\n")
                return ReturnCodes.SUCCESS
        except IdTokenError:
            self.rdmc.ui.printer("Insufficient Privilege to import TLS certificate\n")
            return ReturnCodes.RIS_MISSING_ID_TOKEN
        except ScepenabledError:
            self.rdmc.ui.printer("SCEP is enabled , operation not allowed \n")
            return ReturnCodes.SCEP_ENABLED_ERROR
        except IloLicenseError:
            self.rdmc.ui.error("Error occurred while importing TLS certificate")
            return ReturnCodes.ILO_LICENSE_ERROR
        except IncompatibleiLOVersionError:
            self.rdmc.ui.error("iLO FW version on this server doesnt support this operation")
            return ReturnCodes.INCOMPATIBLE_ILO_VERSION_ERROR

    def importcrlhelper(self, options):
        """Helper function for importing CRL certificate

        :param options: list of options
        :type options: list.
        """
        if not self.rdmc.app.typepath.flagiften:
            raise IncompatibleiLOVersionError("This certificate is not available on this system.")

        select = "HpeCertAuth."
        results = self.rdmc.app.select(selector=select)

        try:
            results = results[0]
        except:
            pass

        if results:
            bodydict = results.resp.dict
        else:
            raise NoContentsFoundForOperationError("Unable to find %s" % select)

        for item in bodydict["Actions"]:
            if "ImportCRL" in item:
                action = item.split("#")[-1]
                path = bodydict["Actions"][item]["target"]
                break

        body = {"Action": action, "ImportUri": options.certfile}
        # self.rdmc.app.post_handler(path, body)
        try:
            result = self.rdmc.app.post_handler(path, body)
            if result.status == 200:
                self.rdmc.ui.printer("Imported the CRL certificate successfully\n")
                return ReturnCodes.SUCCESS
        except IdTokenError:
            self.rdmc.ui.printer("Insufficient Privilege to import CRL certificate\n")
            return ReturnCodes.RIS_MISSING_ID_TOKEN
        except IloLicenseError:
            self.rdmc.ui.error("Error Occured while importing CRL certificate")
            return ReturnCodes.ILO_LICENSE_ERROR
        except IncompatibleiLOVersionError:
            self.rdmc.ui.error("iLO FW version on this server doesnt support this operation")
            return ReturnCodes.INCOMPATIBLE_ILO_VERSION_ERROR

    def importcahelper(self, options):
        """Helper function for importing CA certificate

        :param options: list of options
        :type options: list.
        """
        if not self.rdmc.app.typepath.flagiften:
            raise IncompatibleiLOVersionError("This certificate is not available on this system.")

        tlsfile = options.certfile

        try:
            with open(tlsfile) as certfile:
                certdata = certfile.read()
                certfile.close()
        except:
            raise InvalidFileInputError("Error loading the specified file.")

        select = "HpeCertAuth."
        results = self.rdmc.app.select(selector=select)

        try:
            results = results[0]
        except:
            pass

        if results:
            bodydict = results.resp.dict
        else:
            raise NoContentsFoundForOperationError("Unable to find %s" % select)

        for item in bodydict["Actions"]:
            if "ImportCACertificate" in item:
                action = item.split("#")[-1]
                path = bodydict["Actions"][item]["target"]
                break

        body = {"Action": action, "Certificate": certdata}

        try:
            result = self.rdmc.app.post_handler(path, body)
            if result.status == 200:
                self.rdmc.ui.printer("Imported the CA certificate successfully\n")
                return ReturnCodes.SUCCESS
        except IdTokenError:
            self.rdmc.ui.printer("Insufficient Privilege to import CA certificate\n")
            return ReturnCodes.RIS_MISSING_ID_TOKEN
        except IloLicenseError:
            self.rdmc.ui.error("Error occurred while importing CA certificate")
            return ReturnCodes.ILO_LICENSE_ERROR
        except IncompatibleiLOVersionError:
            self.rdmc.ui.error("iLO FW version on this server doesnt support this operation")
            return ReturnCodes.INCOMPATIBLE_ILO_VERSION_ERROR

    def importplatformhelper(self, options):
        """Helper function for importing a platform certificate
        :param options: options attributes
        :type options: attributes
        """

        # ss_instance = next(
        #     iter(self.rdmc.app.select("SecurityService.", path_refresh=True))
        # )
        if getattr(options, "ldevid_cert"):
            instance_path_uri = "/redfish/v1/Managers/1/Diagnostics/Actions/HpeiLODiagnostics.ImportiLOLDevID"
        else:
            ss_instance = self.rdmc.app.get_handler("/redfish/v1/Managers/1/Diagnostics/", service=True, silent=True)
            instance_path_uri = None

        if getattr(options, "ldevid_cert"):
            type = "iLOLDevID"
        elif getattr(options, "idevid_cert"):
            type = "iLOIDevID"
        elif getattr(options, "systemiak_cert"):
            type = "SystemIAK"
        elif getattr(options, "systemidevid_cert"):
            type = "SystemIDevID"
        elif getattr(options, "platform_cert"):
            type = "PlatformCert"
        else:
            raise InvalidCommandLineErrorOPTS(
                "An invalid set of options were selected...verify " " options were selected correctly and re-try."
            )
        type_totarget = "#HpeiLODiagnostics.Import" + type

        if not instance_path_uri:
            instance_path_uri = ss_instance.dict["Actions"][type_totarget]["target"]

        # instance_path_uri = (
        #     (
        #         ss_instance.dict[type]["Certificates"][
        #             self.rdmc.app.typepath.defs.hrefstring
        #         ]
        #     )
        #     if ss_instance.dict.get("SystemIAK")
        #     else None
        # )
        if instance_path_uri:
            certdata = None
            try:
                with open(options.certfile) as cf:
                    certdata = cf.read()
            except:
                raise InvalidFileInputError("Error loading the specified file.")

            payload = {"Certificate": certdata}

            if certdata:
                try:
                    result = self.rdmc.app.post_handler(instance_path_uri, payload)
                    if result.status == 200:
                        self.rdmc.ui.printer("Imported the %s certificate successfully\n" % type)
                        return ReturnCodes.SUCCESS
                except IdTokenError:
                    self.rdmc.ui.printer("Insufficient Privilege to import %s certificate\n" % type)
                    return ReturnCodes.RIS_MISSING_ID_TOKEN
                except IloLicenseError:
                    self.rdmc.ui.error("Error Occured while importing %s certificate\n" % type)
                    return ReturnCodes.ILO_LICENSE_ERROR
                except IncompatibleiLOVersionError:
                    self.rdmc.ui.error("iLO FW version on this server doesnt support this operation")
                    return ReturnCodes.INCOMPATIBLE_ILO_VERSION_ERROR

    def certificatesvalidation(self, options):
        """certificates validation function

        :param options: command line options
        :type options: list.
        """
        self.cmdbase.login_select_validation(self, options)

    def definearguments(self, customparser):
        """Wrapper function for certificates command main function

        :param customparser: command line input
        :type customparser: parser.
        """
        if not customparser:
            return

        self.cmdbase.add_login_arguments_group(customparser)

        subcommand_parser = customparser.add_subparsers(dest="command")

        # gen csr sub-parser
        gen_csr_help = "Please use the below example command to execute the same function."
        gen_csr_parser = subcommand_parser.add_parser(
            "csr",
            help=gen_csr_help,
            description=gen_csr_help + "\nexample: certificate gen_csr [ORG_NAME] [ORG_UNIT]"
            " [COMMON_NAME] [COUNTRY] [STATE] [CITY]\n\nNOTE: please make "
            "certain the order of arguments is correct.",
            formatter_class=RawDescriptionHelpFormatter,
        )
        gen_csr_parser.add_argument(
            "csr_orgname",
            help="Organization name. i.e. Hewlett Packard Enterprise.",
            metavar="ORGNAME",
        )
        gen_csr_parser.add_argument(
            "csr_orgunit",
            help="Organization unit. i.e. Intelligent Provisioning.",
            metavar="ORGUNIT",
        )
        gen_csr_parser.add_argument(
            "csr_commonname",
            help="Organization common name. i.e. Common Organization Name.",
            metavar="ORGCNAME",
        )
        gen_csr_parser.add_argument(
            "csr_country",
            help="Organization country. i.e. United States.",
            metavar="ORGCOUNTRY",
        )
        gen_csr_parser.add_argument("csr_state", help="Organization state. i.e. Texas.", metavar="ORGSTATE")
        gen_csr_parser.add_argument("csr_city", help="Organization city. i.e. Houston.", metavar="ORGCITY")
        self.cmdbase.add_login_arguments_group(gen_csr_parser)

        # get csr
        get_csr_help = (
            "Retrieve the generated certificate signing request (CSR) printed to the " "console or to a json file."
        )
        get_csr_parser = subcommand_parser.add_parser(
            "getcsr",
            help=get_csr_help,
            description=get_csr_help
            + "\nexample: certificate getcsr\nexample: certificate getcsr  --TLS_CERT/--PLATFORM_CERT"
            "-f mycsrfile.json",
            formatter_class=RawDescriptionHelpFormatter,
        )
        get_csr_parser.add_argument(
            "--TLS_CERT",
            dest="TLSCERT",
            help="specify to retrieve a TLS/SSL certificate signing request.",
            action="store_true",
            default=None,
        )
        get_csr_parser.add_argument(
            "--PLATFORM_CERT",
            dest="PLATFORM",
            help="specify to retrieve a platform certificate signing request.",
            action="store_true",
            default=None,
        )
        get_csr_parser.add_argument(
            "-f",
            "--filename",
            dest="filename",
            help="Use this flag if you wish to use a different"
            " filename for the certificate signing request. The default"
            " filename is %s." % __filename__,
            action="append",
            default=None,
        )
        self.cmdbase.add_login_arguments_group(get_csr_parser)

        # ca certificate
        ca_help = "This command has been deprecated. Please use the below example command to execute the same function."
        ca_parser = subcommand_parser.add_parser(
            "ca",
            help=ca_help,
            description=ca_help + "\nexample: certificate import --ca_cert mycertfile.txt\nNote: The "
            "certificate must be in X.509 format",
            formatter_class=RawDescriptionHelpFormatter,
        )
        ca_parser.add_argument("certfile", help="X.509 formatted CA certificate", metavar="CACERTFILE")
        self.cmdbase.add_login_arguments_group(ca_parser)

        # crl certificate
        crl_help = (
            "This command has been deprecated. Please use the below example command to execute the same function."
        )
        crl_parser = subcommand_parser.add_parser(
            "crl",
            help=crl_help,
            description=crl_help + "\nexample: certificate import --crl_cert https://mycertfileurl/mycertfile.txt"
            "\nNote: The certificate must be in X.509 format",
            formatter_class=RawDescriptionHelpFormatter,
        )
        crl_parser.add_argument(
            "certfile_url",
            help="URL pointing to the location of the X.509 CA certificate",
            metavar="CERTFILEURL",
        )
        self.cmdbase.add_login_arguments_group(crl_parser)

        # tls certificate
        tls_help = (
            "This command has been deprecated. Please use the below example command to execute the same function."
        )
        tls_parser = subcommand_parser.add_parser(
            "tls",
            help=tls_help,
            description=tls_help + "\nexample: certificate import --tls_cert mycertfile.txt\nNote: The "
            "certificate must be in TLS X.509 format",
            formatter_class=RawDescriptionHelpFormatter,
        )
        tls_parser.add_argument("certfile", help="X.509 formatted TLS certificate", metavar="TLSCERTFILE")
        self.cmdbase.add_login_arguments_group(tls_parser)

        # view certificate
        view_help = "View Certificates (https or scep)"
        view_parser = subcommand_parser.add_parser(
            "view",
            help=view_help,
            description=view_help
            + "\nexample: certificate view --https_cert  \n or \n certificate view --scep_cert \n"
            + "Webserver certificate whether self-signed or manually imported or issued by SCEP server can be viewed",
            formatter_class=RawDescriptionHelpFormatter,
        )
        view_parser.add_argument(
            "--scep_cert",
            dest="scep_cert",
            help="Gets the information of SCEP settings for iLO such as SCEP enable status, URL of the SCEP server,"
            + " ChallengePassword, SCEP CA certificate name, webserver CSR subject contents, SCEP enrollment status",
            action="store_true",
        )
        view_parser.add_argument(
            "--https_cert",
            dest="https_cert",
            help="Gets the https certificate whether self-signed or manually imported or issued by SCEP server",
            action="store_true",
        )
        self.cmdbase.add_login_arguments_group(view_parser)

        # import certificate
        import_help = "Imports the Certificates."
        import_parser = subcommand_parser.add_parser(
            "import",
            help=import_help,
            description=import_help
            + "\nexample: certificate import --scep_cert certificate.txt \n"
            + " make sure you are providing a .txt file input.\n",
            formatter_class=RawDescriptionHelpFormatter,
        )
        import_parser.add_argument(
            "--scep_cert",
            dest="scep_cert",
            help="Gets the https certificate whether self-signed or manually imported or issued by SCEP server",
            action="store_true",
        )
        import_parser.add_argument(
            "--ca_cert",
            dest="ca_cert",
            help="Upload a X.509 formatted CA certificate to iLO.",
            action="store_true",
        )
        import_parser.add_argument(
            "--crl_cert",
            dest="crl_cert",
            help="Provide iLO with a URL to retrieve the X.509 formatted CA certificate.",
            action="store_true",
        )
        import_parser.add_argument(
            "--tls_cert",
            dest="tls_cert",
            help="Upload a X.509 TLS certificate to iLO.",
            action="store_true",
        )
        import_parser.add_argument(
            "--idevid_cert",
            dest="idevid_cert",
            help="Upload an IDEVID certificate.",
            action="store_true",
        )
        import_parser.add_argument(
            "--ldevid_cert",
            dest="ldevid_cert",
            help="Upload an LDEVID certificate.",
            action="store_true",
        )
        import_parser.add_argument(
            "--systemiak_cert",
            dest="systemiak_cert",
            help="Upload an System IAK certificate.",
            action="store_true",
        )
        import_parser.add_argument(
            "--systemidevid_cert",
            dest="systemidevid_cert",
            help="Upload an system IDEVID certificate.",
            action="store_true",
        )
        import_parser.add_argument(
            "--platform_cert",
            dest="platform_cert",
            help="Upload a platform certificate.",
            action="store_true",
        )
        import_parser.add_argument(
            "--from_url",
            dest="from_url",
            help="Use this flag to specify a URL for certificate import",
            action="append",
            default=None,
        )
        import_parser.add_argument(
            "certfile",
            help="Certificate can be imported via POST action",
            metavar="certfile",
        )
        self.cmdbase.add_login_arguments_group(import_parser)

        # export cert
        export_help = "Pull an X.509 formatted platform certificate from iLO."
        export_parser = subcommand_parser.add_parser(
            "export",
            help=export_help,
            description=export_help + "\nExample: certificate export --IDEVID -f " "myidevidfile\n",
            formatter_class=RawDescriptionHelpFormatter,
        )

        export_parser.add_argument(
            "--idevid_cert",
            dest="idevid_cert",
            help="Specify for an IDEVID certificate. ",
            action="store_true",
            default=None,
        )
        export_parser.add_argument(
            "--ldevid_cert",
            dest="ldevid_cert",
            help="Specify for an LDEVID certificate. ",
            action="store_true",
            default=None,
        )
        export_parser.add_argument(
            "--systemiak_cert",
            dest="systemiak_cert",
            help="Specify for a system IAK certificate.",
            action="store_true",
            default=None,
        )
        export_parser.add_argument(
            "--systemidevid_cert",
            dest="systemidevid_cert",
            help="Specify for a system IDEVID certificate.",
            action="store_true",
            default=None,
        )
        export_parser.add_argument(
            "--platform_cert",
            dest="platform_cert",
            help="Specify for a platform certificate.",
            action="store_true",
            default=None,
        )
        export_parser.add_argument(
            "--id",
            dest="id",
            help="Optionally specify the certificate instance, if multiples are available. If"
            "the instance specified is not available, then the next is retrieved. Default is"
            "the first instance",
            default=1,
        )
        export_parser.add_argument(
            "-f",
            "--filename",
            dest="filename",
            help="Use this flag to import a certificate from or export a certificate to a file.",
            action="append",
            default=None,
        )

        self.cmdbase.add_login_arguments_group(export_parser)

        # delete certificate
        delete_help = "Deletes the https Certificate"
        delete_parser = subcommand_parser.add_parser(
            "delete",
            help=delete_help,
            description=delete_help + "\nexample: certificate delete \n  delete the https_cert certificate ",
            formatter_class=RawDescriptionHelpFormatter,
        )

        self.cmdbase.add_login_arguments_group(delete_parser)

        # gen csr sub-parser
        gencsr_help = (
            "Generate a certificate signing request (CSR) for iLO SSL certificate "
            "authentication.\nNote: iLO will create a Base64 encoded CSR in PKCS "
            "#10 Format."
        )
        gencsr_parser = subcommand_parser.add_parser(
            "gen_csr",
            help=gencsr_help,
            description=gen_csr_help + "\nexample: certificate gen_csr [ORG_NAME] [ORG_UNIT]"
            " [COMMON_NAME] [COUNTRY] [STATE] [CITY] [INCLUDEIP] \n\nNOTE: please make "
            "certain the order of arguments is correct.",
            formatter_class=RawDescriptionHelpFormatter,
        )
        gencsr_parser.add_argument(
            "gencsr_orgname",
            help="Organization name. i.e. Hewlett Packard Enterprise.",
            metavar="ORGNAME",
        )
        gencsr_parser.add_argument(
            "gencsr_orgunit",
            help="Organization unit. i.e. Intelligent Provisioning.",
            metavar="ORGUNIT",
        )
        gencsr_parser.add_argument(
            "gencsr_commonname",
            help="Organization common name. i.e. Common Organization Name.",
            metavar="ORGCNAME",
        )
        gencsr_parser.add_argument(
            "gencsr_country",
            help="Organization country. i.e. United States.",
            metavar="ORGCOUNTRY",
        )
        gencsr_parser.add_argument("gencsr_state", help="Organization state. i.e. Texas.", metavar="ORGSTATE")
        gencsr_parser.add_argument("gencsr_city", help="Organization city. i.e. Houston.", metavar="ORGCITY")
        gencsr_parser.add_argument(
            "gencsr_parser_includeIP",
            help="Include IP. i.e. True or False.",
            metavar="INCLUDEIP",
        )
        self.cmdbase.add_login_arguments_group(gencsr_parser)

        # automatic enrollment sub-parser
        autoenroll_help = (
            "Use this command for invoking the auto enroll the certificate enrollment process"
            "\nMake sure you have imported the scep CA certificate "
            "before head using certificate import --scep certifcate.txt"
        )
        autoenroll_parser = subcommand_parser.add_parser(
            "auto_enroll",
            help=autoenroll_help,
            description=autoenroll_help + "\nexample: certificate auto_enroll [ORG_NAME] [ORG_UNIT]"
            " [COMMON_NAME] [COUNTRY] [STATE] [CITY] [SCEP_ADDRESS] [CHALLENGEPASSWORD] "
            "[SERVICEENABLED] [INCLUDEIP]]\n\nNOTE: please make "
            "certain the order of arguments is correct.",
            formatter_class=RawDescriptionHelpFormatter,
        )
        autoenroll_parser.add_argument(
            "autoenroll_orgname",
            help="Organization name. i.e. Hewlett Packard Enterprise.",
            metavar="ORGNAME",
        )
        autoenroll_parser.add_argument(
            "autoenroll_orgunit",
            help="Organization unit. i.e. Intelligent Provisioning.",
            metavar="ORGUNIT",
        )
        autoenroll_parser.add_argument(
            "autoenroll_commonname",
            help="Organization common name. i.e. Common Organization Name.",
            metavar="ORGCNAME",
        )
        autoenroll_parser.add_argument(
            "autoenroll_country",
            help="Organization country. i.e. United States.",
            metavar="ORGCOUNTRY",
        )
        autoenroll_parser.add_argument("autoenroll_state", help="Organization state. i.e. Texas.", metavar="ORGSTATE")
        autoenroll_parser.add_argument("autoenroll_city", help="Organization city. i.e. Houston.", metavar="ORGCITY")
        autoenroll_parser.add_argument(
            "autoenroll_scep_enrollAddress",
            help="Scep-enroll dll address",
            metavar="AEADDRESS",
        )
        autoenroll_parser.add_argument(
            "autoenroll_challengepassword",
            help="challenge password",
            metavar="AECHALLPASS",
        )
        autoenroll_parser.add_argument(
            "autoenroll_ScepService",
            help="Scep service enable or disable",
            metavar="AESCEPSERVICE",
        )
        autoenroll_parser.add_argument(
            "autoenroll_includeIP",
            help="Include IP. i.e. True or False.",
            metavar="INCLUDEIP",
        )
        self.cmdbase.add_login_arguments_group(autoenroll_parser)



"""
This module contains tests for ShowPmemCommand
"""
from __future__ import absolute_import

import pytest

from rdmc_base_classes import RdmcCommandBase
from rdmc_helper import InvalidCommandLineError

from ..conftest import ShowPmemCommand


class CustomOptions(object):
    """
    Class to mock different options
    """

    def __init__(self):
        self.all = True
        self.display_attributes = None
        self.json = False

    def all_flag(self):
        """
        Function to set --all flag to true
        :return: CustomOptions object
        """
        self.all = True
        return self

    def no_flag(self):
        """
        Function to reset flags
        :return: CustomOptions object
        """
        self.all = False
        self.display_attributes = None
        return self

    def display_flag(self):
        """
        Function to set --display flag
        :return: CustomOptions object
        """
        self.all = False
        self.display_attributes = ["RandomNestedKey1"]
        return self

    def wrong_display_flag(self):
        """
        Function to set --display flag with wrong attributes
        :return: CustomOptions object
        """
        self.all = False
        self.display_attributes = ["aBc", "xYz"]
        return self

    @staticmethod
    def get_default(flag):
        """
        Function to return default attributes
        :return: list of attributes
        """
        if flag == "device":
            return ["Location", "SizeMiB"]
        if flag == "config":
            return ["Location", "Capacity"]
        return []

    @staticmethod
    def get_all(flag):
        """
        Function to return attributes for --all flag
        :return: list of attributes
        """
        if flag == "device":
            return ["Location", "SizeMiB", "RandomNestedKey1"]
        if flag == "config":
            return ["Location", "Capacity", "RandomNestedKey1"]
        return []


class MockMappingTable(object):
    """
    class to mock mapping table
    """
    memory = {'Capacity': {'path': '/CapacityMiB', 'redfishName': 'CapacityMiB'},
              'Location': {'path': '/DeviceLocator', 'redfishName': 'DeviceLocator'},
              'RandomAttribute': {'path': "/RandomAttribute",
                                  'redfishName': 'RandomAttribute'},
              'RandomNestedKey1': {'path': "/RandomAttribute/RandomNestedKey1",
                                   'redfishName': 'RandomNestedKey1'},
              'SizeMiB': {'path': '/Regions/SizeMiB', 'redfishName': 'SizeMiB'}}
    memorychunk = {'Capacity': {'path': '/CapacityMiB', 'redfishName': 'CapacityMiB'},
                   'Location': {'path': '/DeviceLocator', 'redfishName': 'DeviceLocator'},
                   'RandomAttribute': {'path': "/RandomAttribute",
                                       'redfishName': 'RandomAttribute'},
                   'RandomNestedKey1': {'path': "/RandomAttribute/RandomNestedKey1",
                                        'redfishName': 'RandomNestedKey1'},
                   'SizeMiB': {'path': '/Regions/SizeMiB', 'redfishName': 'SizeMiB'}}


class TestShowPmemCommands(object):
    """
    Class to test functions in ShowPmemCommands
    """
    mock_data = [
        {
            'DeviceLocator': 'PROC 1 DIMM 1',
            'CapacityMiB': 123456,
            'SizeMiB': 'xyz',
            'RandomAttribute': {
                'RandomNestedKey1': 'RandomNestedValue1.1',
                'RandomNestedKey2': [
                    'RandomNestedValue2.1',
                    'RandomNestedValue2.2'
                ],
                'RandomNestedKey4': [
                    {
                        'RandomNestedKey5': 'RandomNestedValue5.1'
                    }
                ]
            }
        },
        {
            'DeviceLocator': 'PROC 2 DIMM 12',
            'CapacityMiB': 987654,
            'SizeMiB': 'abc@123',
            'RandomAttribute': {
                'RandomNestedKey1': 'RandomNestedValue1.2',
                'RandomNestedKey2': [
                    'RandomNestedValue2.3',
                    'RandomNestedValue2.4'
                ],
                'RandomNestedKey4': [
                    {
                        'RandomNestedKey5': 'RandomNestedValue5.2'
                    }
                ]
            }
        }]

    custom_options_obj = CustomOptions()

    @pytest.mark.parametrize('members, options, flag, expected_output', [
        # display flag
        (mock_data, CustomOptions().display_flag(), "device",
         ['\nRandomNestedKey1: RandomNestedValue1.1', '\nRandomNestedKey1: RandomNestedValue1.2']),
        (mock_data, CustomOptions().display_flag(), "config",
         ['\nRandomNestedKey1: RandomNestedValue1.1', '\nRandomNestedKey1: RandomNestedValue1.2']),
        # no flag
        ([mock_data[1]], CustomOptions().no_flag(), "device",
         ['\nLocation: PROC 2 DIMM 12\nSizeMiB: None']),
        ([mock_data[0]], CustomOptions().no_flag(), "config",
         ['\nLocation: PROC 1 DIMM 1\nCapacity: 123456'])
    ])
    @pytest.mark.skip(reason="temporarily remove display flag")
    def test_generate_display_output_pass(self, members, options, flag, expected_output,
                                          monkeypatch):
        """
        Tests for generate_display_output() with valid input
        :param members: list of members returned as a result of GET request
        :param options: command options
        :param flag: name of flag
        :param expected_output: expected output of the test
        """
        monkeypatch.setattr(ShowPmemCommand, "get_default_attributes",
                            self.custom_options_obj.get_default)
        pmem_command = ShowPmemCommand.ShowPmemCommand(RdmcCommandBase)
        result = pmem_command.generate_display_output(members, options, flag,
                                                      MockMappingTable.memory)
        assert result == expected_output

    @staticmethod
    @pytest.mark.parametrize('members, options, flag, expected_output', [
        ([mock_data[0]], CustomOptions().wrong_display_flag(), "device",
         ["\nNo match found for 'aBc'\nNo match found for 'xYz'"]),
        ([mock_data[1]], CustomOptions().wrong_display_flag(), "config",
         ["\nNo match found for 'aBc'\nNo match found for 'xYz'"]),
        (mock_data, CustomOptions().wrong_display_flag(), "config",
         ["\nNo match found for 'aBc'\nNo match found for 'xYz'",
          "\nNo match found for 'aBc'\nNo match found for 'xYz'"]),
    ])
    @pytest.mark.skip(reason="temporarily remove display flag")
    def test_generate_display_output_fail(members, options, flag, expected_output):
        """
        Tests for generate_display_output() with invalid input
        :param members: list of members returned as a result of GET request
        :param options: command options
        :param flag: name of flag
        :param expected_output: expected output of the test
        """
        pmem_command = ShowPmemCommand.ShowPmemCommand(RdmcCommandBase)
        result = pmem_command.generate_display_output(members, options, flag,
                                                      MockMappingTable.memory)
        assert result == expected_output

    @staticmethod
    @pytest.mark.parametrize('customparser, expected_output', [
        ('', None)
    ])
    def test_define_arguments(customparser, expected_output):
        """
        Branch Test for definearguments() when parser is not customparser
        :param customparser: command line input
        :param expected_output: expected output of the test
        """
        pmem_command = ShowPmemCommand.ShowPmemCommand(RdmcCommandBase)
        result = pmem_command.definearguments(customparser)
        assert result == expected_output

    @staticmethod
    @pytest.mark.parametrize("flag, expected_output", [
        ("device", ShowPmemCommand.DefaultAttributes.device.value),
        ("config", ShowPmemCommand.DefaultAttributes.config.value),
        ("summary", None),
        ("abc", None)])
    def test_get_default_attributes(flag, expected_output):
        """
        Test for get_default_attributes()
        :param flag: name of flag
        :param expected_output: expected output of the test
        """
        result = ShowPmemCommand.get_default_attributes(flag)
        assert result == expected_output


class MockOptions(object):
    """
    Class containing mock options for validation tests
    """
    device = False
    config = False
    summary = False
    logical = False
    dimm = None

    @staticmethod
    def set_options(device, config, logical, summary, dimm):
        """
        Setter method for Mock options
        """
        MockOptions.device = device
        MockOptions.config = config
        MockOptions.logical = logical
        MockOptions.summary = summary
        MockOptions.dimm = dimm


class TestValidation(object):
    """
    Class containing validation tests
    """
    @staticmethod
    @pytest.mark.parametrize("device, config, logical, summary, dimm, expected_result", [
        (False, False, False, False, None, None),
        (False, False, False, False, ["1@1", "2@10"], None),
        (True, False, False, False, None, None),
        (True, False, False, False, ["3@12", "4@8"], None),
        (False, True, False, False, None, None),
        (True, False, False, False, ["4@4", "3@3"], None),
        (True, False, False, False, ["9@19"], None),
        (False, False, True, False, None, None),
        (False, False, False, True, None, None)
    ])
    def test_validate_show_pmem_options_pass(device, config, logical,
                                             summary, dimm, expected_result):
        """
        Tests for 'validate_options' with valid options
        """
        MockOptions.set_options(device, config, logical, summary, dimm)
        pmem_command = ShowPmemCommand.ShowPmemCommand(RdmcCommandBase)
        result = pmem_command.validate_show_pmem_options(MockOptions)
        assert result == expected_result

    @staticmethod
    @pytest.mark.parametrize("device, config, logical, summary, dimm", [
        (True, True, True, True, None),
        (True, True, True, False, None),
        (True, True, False, True, None),
        (True, False, True, True, None),
        (False, True, True, True, None),
        (True, True, False, False, None),
        (True, False, True, False, None),
        (True, False, False, True, None),
        (False, False, True, True, None),
        (False, True, False, True, None),
        (False, True, True, False, None),
        (True, False, False, False, ["0@0"]),
        (True, False, False, False, ["0@8"]),
        (True, False, False, False, ["4@0"]),
        (True, False, False, False, ["4@20"]),
        (True, False, False, False, ["10@20"]),
        (True, False, False, False, ["10@2"]),
        (True, False, False, False, ["3)12", "438"]),
        (True, False, False, False, ["4#4", "3!3"]),
        (True, False, False, False, ["3)12", "438"]),
        (False, False, False, False, ["1@1", "2@=10"]),
        (False, False, True, False, ["3@12", "4@8"]),
        (False, False, False, True, ["4@4", "3@3"])
    ])
    def test_validate_show_pmem_options_fail(device, config, logical, summary, dimm):
        """
        Tests for 'validate_options' with invalid options
        """
        MockOptions.set_options(device, config, logical, summary, dimm)
        pmem_command = ShowPmemCommand.ShowPmemCommand(RdmcCommandBase)
        with pytest.raises(InvalidCommandLineError):
            pmem_command.validate_show_pmem_options(MockOptions)

    @staticmethod
    @pytest.mark.parametrize("device, config, logical, summary, dimm", [
        (False, False, False, False, None),
        (False, False, False, False, ["1@2"]),
        (True, False, False, False, None),
        (True, False, False, False, ["4#4", "3!3"]),
        (False, True, False, False, None),
        (False, True, False, False, ["4@4", "3@3"]),
        (False, False, True, False, None),
        (False, False, True, False, ["1@1", "2@=10"]),
        (False, False, False, True, None),
        (False, False, False, True, ["1@1", "2@10"])
    ])
    def test_validate_args(device, config, logical, summary, dimm):
        """
        Tests for 'validate_args'
        """
        MockOptions.set_options(device, config, logical, summary, dimm)
        pmem_command = ShowPmemCommand.ShowPmemCommand(RdmcCommandBase)
        with pytest.raises(InvalidCommandLineError):
            pmem_command.validate_args(MockOptions)
